@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2000-2006 Julien Pommier
+// Copyright (C) 2000-2007 Julien Pommier
 //
 // This file is a part of GETFEM++
 //
@@ -21,8 +21,8 @@
 //========================================================================
 
 #include <bitset>
-#include "gmm_blas_interface.h"
-#include "bgeot_sparse_tensors.h"
+#include "gmm/gmm_blas_interface.h"
+#include "getfem/bgeot_sparse_tensors.h"
 
 
 namespace bgeot {
@@ -309,12 +309,12 @@ namespace bgeot {
   }
 
   void tensor_mask::check_assertions() const {
-    if (r.size() != idxs.size()) DAL_INTERNAL_ERROR("");
-    if (s.size() != idxs.size()+1) DAL_INTERNAL_ERROR("");
-    if (m.size() != size()) DAL_INTERNAL_ERROR("");
+    GMM_ASSERT3(r.size() == idxs.size(), "");
+    GMM_ASSERT3(s.size() == idxs.size()+1, "");
+    GMM_ASSERT3(m.size() == size(), "");
     dal::bit_vector bv;
     for (dim_type i=0; i < idxs.size(); ++i) {
-      if (bv.is_in(i)) DAL_INTERNAL_ERROR(""); 
+      GMM_ASSERT3(!bv.is_in(i), "");
       bv.add(i);
     }
   }
@@ -600,7 +600,8 @@ namespace bgeot {
       if (vectorized_pr_dim == pri.size()-1) {
         if (pp->have_regular_strides.count() == N) vectorized_size_ = pp->range;
         for (dim_type n=pp->n; n < N; ++n) {
-          vectorized_strides_[n] = pp->inc[n];
+	  GMM_ASSERT3(n < pp->inc.size(), ""); // expected failure here with "empty" tensors, see tests/test_assembly.cc, testbug() function.
+          vectorized_strides_[n] = pp->inc[n]; 
         }
       } else {
         if (pp->have_regular_strides.count() != N) break;
@@ -711,8 +712,8 @@ namespace bgeot {
     update_reduction_chars();
 
     //cout << "find_best_reduction: reduction_chars='" << reduction_chars << "'\n";
-    if (trtab.size() > 32) 
-      DAL_INTERNAL_ERROR("wow it was assumed that nobody would ever need a reduction on more than 32 tensors..");
+    GMM_ASSERT2(trtab.size() <= 32, "wow it was assumed that nobody would "
+		"ever need a reduction on more than 32 tensors..");
 
     std::vector<std::bitset<32> > idx_occurences(reduction_chars.size());
 
@@ -830,10 +831,9 @@ namespace bgeot {
       pout_data = &out_data[0];
       trres.set_base(pout_data);
     } else {
-      if (tr_out->ndim() != reduced_range.size()) 
-	DAL_INTERNAL_ERROR("");
-      for (dim_type i=0; i < tr_out->ndim(); ++i) if (tr_out->dim(i) != reduced_range[i]) 
-	DAL_INTERNAL_ERROR("");
+      GMM_ASSERT3(tr_out->ndim() == reduced_range.size(), "");
+      for (dim_type i=0; i < tr_out->ndim(); ++i)
+	GMM_ASSERT3(tr_out->dim(i) == reduced_range[i], "");
       trres = *tr_out;
     }
 
@@ -860,9 +860,8 @@ namespace bgeot {
             global_range.push_back(it->tr().dim(i));
             it->gdim[i] = global_range.size() - 1;
           } else {
-            if (it->tr().dim(i) != global_range[p]) 
-              DAL_THROW(std::invalid_argument, 
-                        "inconsistent dimensions for reduction index " 
+	    GMM_ASSERT1(it->tr().dim(i) == global_range[p],
+			"inconsistent dimensions for reduction index " 
                         << it->ridx[i] << "(" << int(it->tr().dim(i)) 
                         << " != " << int(global_range[p]) << ")");
             it->gdim[i] = p;
@@ -1035,7 +1034,7 @@ namespace bgeot {
 	mti.p(0) += s1;
       } while (mti.bnext(0));	
     } else {
-      DAL_THROW(std::invalid_argument, "unhandled reduction case ! (N=" << int(N) << ")");
+      GMM_ASSERT1(false, "unhandled reduction case ! (N=" << int(N) << ")");
     }
   }
 
