@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2003-2006 Julien Pommier
+// Copyright (C) 2003-2007 Julien Pommier
 //
 // This file is a part of GETFEM++
 //
@@ -20,8 +20,8 @@
 //
 //========================================================================
 
-#include <getfem_assembling_tensors.h>
-#include <getfem_mat_elem.h>
+#include "getfem/getfem_assembling_tensors.h"
+#include "getfem/getfem_mat_elem.h"
 
 extern "C" void daxpy_(const int *n, const double *alpha, const double *x, const int *incx, double *y, const int *incy);
 
@@ -896,7 +896,7 @@ namespace getfem {
         do_post_reduction(cv);
         data_base = &fallback_red.out_data[0];
       } else data_base = &(*t.begin());
-      if (t.size() != size_type(tsize)) DAL_INTERNAL_ERROR(t.size() << " != " << tsize);
+      GMM_ASSERT3(t.size() == size_type(tsize), "");
     }
   };
 
@@ -906,6 +906,8 @@ namespace getfem {
     const base_asm_data *basm; //scalar_type* global_array;
     vdim_specif_list vdim;
     multi_tensor_iterator mti;
+    tensor_ranges e_r;
+    std::vector< tensor_strides > e_str;
   public:
     ATN_tensor_from_dofs_data(const base_asm_data *basm_, 
 			      const vdim_specif_list& d) :
@@ -934,12 +936,10 @@ namespace getfem {
 
   private:
     void exec_(size_type cv, dim_type ) {
-      tensor_ranges r;
-      std::vector< tensor_strides > str;
-      vdim.build_strides_for_cv(cv, r, str);
-      assert(r == ranges());
+      vdim.build_strides_for_cv(cv, e_r, e_str);
+      assert(e_r == ranges());
       mti.rewind();
-      basm->copy_with_mti(str, mti);
+      basm->copy_with_mti(e_str, mti);
     }
   };
   
@@ -1611,7 +1611,7 @@ namespace getfem {
     case wALIAS: {
       vars[ident] = t.tensor(); t.tensor()->freeze();
     } break;
-    default: DAL_INTERNAL_ERROR(""); break;
+    default: GMM_ASSERT3(false, ""); break;
     }
     pop_mark();
   }
@@ -1690,55 +1690,56 @@ namespace getfem {
   }
 
   struct cv_fem_compare {
-    const std::deque<const mesh_fem *>& mf;
-    cv_fem_compare(const std::deque<const mesh_fem *>& mf_) : mf(mf_) {}
-    bool operator()(size_type a, size_type b) {
+    const std::vector<const mesh_fem *> &mf;
+    cv_fem_compare(const std::vector<const mesh_fem *>& mf_) : mf(mf_) {}
+    bool operator()(size_type a, size_type b) const {
       for (size_type i=0; i < mf.size(); ++i) {
-	/*
-	  if (mf[i]->fem_of_element(a) < mf[i]->fem_of_element(b))
-	  return true;
-	if (mf[i]->fem_of_element(a) == mf[i]->fem_of_element(b) &&
-	    mf[i]->nb_dof_of_element(a) < mf[i]->nb_dof_of_element(b))
-	  return true;
-	*/
-
+	pfem pfa(mf[i]->fem_of_element(a));
+	pfem pfb(mf[i]->fem_of_element(b));
 	/* sort by nb_dof and then by fem */
-	if (mf[i]->nb_dof_of_element(a) < mf[i]->nb_dof_of_element(b)) {
+	unsigned nba = pfa->nb_dof(a);
+	unsigned nbb = pfb->nb_dof(b);
+	if (nba < nbb) {
 	  return true;
-	}
-	if (mf[i]->nb_dof_of_element(a) == mf[i]->nb_dof_of_element(b) && 
-	    mf[i]->fem_of_element(a) < mf[i]->fem_of_element(b)) {
+	} else if (nba > nbb) {
+	  return false;
+	} else if (pfa < pfb) {
 	  return true;
 	}
       }
       return false;
     }
   };
+
   /* reorder the convexes in order to minimize the number of
      shape modifications during the assembly (since this can be
      very expensive) */
   static void get_convex_order(const dal::bit_vector& cvlst,
-			       const std::deque<const mesh_im *>& imtab, 
-			       const std::deque<const mesh_fem *>& mftab, 
+			       const std::vector<const mesh_im *>& imtab, 
+			       const std::vector<const mesh_fem *>& mftab, 
 			       const dal::bit_vector& candidates, 
 			       std::vector<size_type>& cvorder) {
     cvorder.reserve(candidates.card()); cvorder.resize(0);
+    
     for (dal::bv_visitor cv(candidates); !cv.finished(); ++cv) {
       if (cvlst.is_in(cv) && imtab[0]->int_method_of_element(cv) != im_none()) {
 	bool ok = true;
-        for (size_type i=0; i < mftab.size(); ++i)
+        for (size_type i=0; i < mftab.size(); ++i) {
           if (!mftab[i]->convex_index().is_in(cv)) {
 	    ok = false;
             //ASM_THROW_ERROR("the convex " << cv << " has no FEM for the #" << i+1 << " mesh_fem");	  
 	  }
-        if (ok) cvorder.push_back(cv);
+	}
+        if (ok) {
+	  cvorder.push_back(cv);
+	}
       } else if (!imtab[0]->linked_mesh().convex_index().is_in(cv)) {
         ASM_THROW_ERROR("the convex " << cv << " is not part of the mesh");
       } else {
 	/* skip convexes without integration method */
       }
     }
-    std::sort(cvorder.begin(), cvorder.end(), cv_fem_compare(mftab));
+    //std::sort(cvorder.begin(), cvorder.end(), cv_fem_compare(mftab));
   }
   
   void generic_assembly::consistency_check() {
@@ -1767,7 +1768,7 @@ namespace getfem {
       mesh_region::face_bitset nf = r[cv[i]];
       dim_type f = dim_type(-1);
       while (nf.any()) {
-	// cerr << "generic_assembly::exec(" << cv[i] << ")\n";
+	//cerr << "generic_assembly::exec(" << cv[i] << ")\n";
 	if (nf[0]) exec(cv[i],f);
 	nf >>= 1; f+=1;
       }
