@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2000-2006 Julien Pommier
+// Copyright (C) 2000-2007 Julien Pommier
 //
 // This file is a part of GETFEM++
 //
@@ -24,15 +24,13 @@
 #include <iomanip>
 #include <fstream>
 
-#include <getfem_mesh.h>
-#include <getfem_import.h>
-#include <getfem_regular_meshes.h>
+#include "getfem/getfem_mesh.h"
+#include "getfem/getfem_import.h"
+#include "getfem/getfem_regular_meshes.h"
 
 namespace getfem {
 
-  /* mesh file from gmsh [http://www.geuz.org/gmsh/] 
-     structure: $NOD list_of_nodes $ENDNOD $ELT list_of_elt $ENDELT
-  */
+  /* mesh file from gmsh [http://www.geuz.org/gmsh/]*/
 
   struct gmsh_cv_info {
     unsigned id, type, region;
@@ -40,44 +38,107 @@ namespace getfem {
     std::vector<size_type> nodes;
     void set_pgt() {
       switch (type) {
-	case 1: { /* LINE */
-	  pgt = bgeot::simplex_geotrans(1,1);
-	} break;
-	case 2: { /* TRIANGLE */
-	  pgt = bgeot::simplex_geotrans(2,1);
-	} break;
-	case 3: { /* QUADRANGLE */
-	  pgt = bgeot::parallelepiped_geotrans(2,1);
-	} break;
-	case 4: { /* TETRAHEDRON */
-	  pgt = bgeot::simplex_geotrans(3,1);
-	} break;
-	case 5: { /* HEXAHEDRON */
-	  pgt = bgeot::parallelepiped_geotrans(3,1);
-	} break;
-	case 6: { /* PRISM */
-	  pgt = bgeot::prism_geotrans(3,1);
-	} break;
-	case 7: { /* PYRAMID */
-	  DAL_THROW(failure_error, "sorry pyramidal convexes not done for the moment..");
-	} break;
-	case 15: { /* POINT */
-	  DAL_WARNING2("ignoring point element");
-	} break;
-	default: { /* UNKNOWN .. */
-	  /* second order elements : to be done .. */
-	  DAL_THROW(failure_error, "the gmsh element type " << type << "is unknown..");
-	} break;
+      case 1: { /* LINE */
+	pgt = bgeot::simplex_geotrans(1,1);
+      } break;
+      case 2: { /* TRIANGLE */
+	pgt = bgeot::simplex_geotrans(2,1);
+      } break;
+      case 3: { /* QUADRANGLE */
+	pgt = bgeot::parallelepiped_geotrans(2,1);
+      } break;
+      case 4: { /* TETRAHEDRON */
+	pgt = bgeot::simplex_geotrans(3,1);
+      } break;
+      case 5: { /* HEXAHEDRON */
+	pgt = bgeot::parallelepiped_geotrans(3,1);
+      } break;
+      case 6: { /* PRISM */
+	pgt = bgeot::prism_geotrans(3,1);
+      } break;
+      case 7: { /* PYRAMID */
+	GMM_ASSERT1(false, "sorry pyramidal elements not yet supported.");
+      } break;
+      case 8: { /* 2ND ORDER LINE */
+	pgt = bgeot::simplex_geotrans(1,2);
+      } break;
+      case 9: { /* 2ND ORDER TRIANGLE */
+	pgt = bgeot::simplex_geotrans(2,2);
+      } break;
+      case 15: { /* POINT */
+	GMM_WARNING2("ignoring point element");
+      } break;
+      default: { /* UNKNOWN .. */
+	/* higher order elements : to be done .. */
+	GMM_ASSERT1(false, "the gmsh element type "<< type <<"is unknown..");
+      } break;
       }
     }
+    
+    void set_nb_nodes() {
+      /* Especially for the gmsh file format version 2.*/
+      switch (type) {
+      case 1: { /* LINE */
+	nodes.resize(2);
+      } break;
+      case 2: { /* TRIANGLE */
+	nodes.resize(3);
+      } break;
+      case 3: { /* QUADRANGLE */
+	nodes.resize(4);
+      } break;
+      case 4: { /* TETRAHEDRON */
+	nodes.resize(4);
+      } break;
+      case 5: { /* HEXAHEDRON */
+	nodes.resize(8);
+      } break;
+      case 6: { /* PRISM */
+	nodes.resize(6);
+      } break;
+      case 7: { /* PYRAMID */
+	GMM_ASSERT1(false,
+		    "sorry pyramidal convexes not done for the moment..");
+      } break;
+      case 8: { /* 2ND ORDER LINE */
+	nodes.resize(3);
+      } break;
+      case 9: { /* 2ND ORDER TRIANGLE */
+	nodes.resize(6);
+      } break;
+      case 15: { /* POINT */
+	GMM_WARNING2("ignoring point element");
+      } break;
+      default: { /* UNKNOWN .. */
+	/* higher order elements : to be done .. */
+	GMM_ASSERT1(false, "the gmsh element type " << type << "is unknown..");
+      } break;
+      }
+    }
+
     bool operator<(const gmsh_cv_info& other) const {
       return pgt->dim() > other.pgt->dim();
     }
   };
 
-  static void import_gmsh_msh_file(std::ifstream& f, mesh& m) {
+  /* 
+     Format version 1 [for gmsh version < 2.0].
+     structure: $NOD list_of_nodes $ENDNOD $ELT list_of_elt $ENDELT
+
+     Format version 2 [for gmsh version >= 2.0]. Modification of some keywords
+     and more than one tag is authorized for a specific region.
+
+     structure: $Nodes list_of_nodes $EndNodes $Elements list_of_elt
+     $EndElements
+  */
+  static void import_gmsh_msh_file(std::ifstream& f, mesh& m, int version = 1)
+  {
+
     /* read the node list */
-    ftool::read_until(f, "$NOD");
+    if (version == 2)
+      bgeot::read_until(f, "$Nodes"); /* Format version 2 */
+    else
+      bgeot::read_until(f, "$NOD");
     
     size_type nb_node; 
     f >> nb_node;
@@ -89,36 +150,93 @@ namespace getfem {
       f >> node_id >> n[0] >> n[1] >> n[2];
       msh_node_2_getfem_node.add_to_index(m.add_point(n), node_id);
     }
-    ftool::read_until(f, "$ENDNOD");
+
+    if (version == 2)
+      bgeot::read_until(f, "$Endnodes"); /* Format version 2 */
+    else
+      bgeot::read_until(f, "$ENDNOD");
     
     /* read the convexes */
-    ftool::read_until(f, "$ELM");
+    if (version == 2)
+      bgeot::read_until(f, "$Elements"); /* Format version 2 */	
+    else
+      bgeot::read_until(f, "$ELM");
+
     size_type nb_cv;
     f >> nb_cv;
-    std::vector<gmsh_cv_info> cvlst(nb_cv);
+    std::vector<gmsh_cv_info> cvlst; cvlst.reserve(nb_cv);
     for (size_type cv=0; cv < nb_cv; ++cv) {
-      gmsh_cv_info &ci = cvlst[cv];
+      unsigned id, type, region;
       unsigned dummy, cv_nb_nodes;
-      f >> ci.id >> ci.type >> ci.region >> dummy >> cv_nb_nodes;
-      ci.id--; /* numbering starts at 1 */
-      ci.nodes.resize(cv_nb_nodes);
-      for (size_type i=0; i < cv_nb_nodes; ++i) {
-	size_type j;
-	f >> j;
-	ci.nodes[i] = msh_node_2_getfem_node.search(j);
-	if (ci.nodes[i] == size_type(-1)) 
-	  DAL_THROW(failure_error, "Invalid node ID " << j 
-		    << " in gmsh convex " << ci.id);
+
+      if (version == 2) { /* Format version 2 */	
+	unsigned nbtags, mesh_part;
+	f >> id >> type >> nbtags;
+	if (nbtags != 3) 
+	  GMM_ASSERT1(false, "Number of tags" << nbtags 
+		      << " is not managed.");      
+	f >> region >> dummy >> mesh_part;
+      }
+      else
+	f >> id >> type >> region >> dummy >> cv_nb_nodes;
+
+      id--; /* numbering starts at 1 */
+      if (type == 15) { // just a node
+        // get rid of the rest of the line
+        f >> dummy;
+      } else {
+        cvlst.push_back(gmsh_cv_info());
+        gmsh_cv_info &ci = cvlst.back();
+        ci.id = id; ci.type = type; ci.region = region;
+
+	if (version == 2) { /* For version 2 */
+	  ci.set_nb_nodes();
+	  cv_nb_nodes = ci.nodes.size();
+	}
+	else
+	  ci.nodes.resize(cv_nb_nodes);
+
+        for (size_type i=0; i < cv_nb_nodes; ++i) {
+          size_type j;
+          f >> j;
+          ci.nodes[i] = msh_node_2_getfem_node.search(j);
+          GMM_ASSERT1(ci.nodes[i] != size_type(-1), "Invalid node ID " << j 
+                      << " in gmsh convex " << ci.id);
+        }
+        ci.set_pgt();
+	if (ci.type == 8) { /* Second order line */
+	  std::vector<size_type> tmp_nodes(3);
+	  tmp_nodes[0] = ci.nodes[0], tmp_nodes[1] = ci.nodes[2],
+	    tmp_nodes[2] = ci.nodes[1];
+	  
+	  ci.nodes[0] = tmp_nodes[0], ci.nodes[1] = tmp_nodes[1],
+	    ci.nodes[2] = tmp_nodes[2];
+	}
+	else if (ci.type == 9)  { /* Second order triangle */
+	  std::vector<size_type> tmp_nodes(6);
+	  tmp_nodes[0] = ci.nodes[0], tmp_nodes[1] = ci.nodes[3],
+	    tmp_nodes[2] = ci.nodes[1];
+	  tmp_nodes[3] = ci.nodes[5], tmp_nodes[4] = ci.nodes[4],
+	    tmp_nodes[5] = ci.nodes[2];
+
+	  ci.nodes[0] = tmp_nodes[0], ci.nodes[1] = tmp_nodes[1],
+	    ci.nodes[2] = tmp_nodes[2];
+	  ci.nodes[3] = tmp_nodes[3], ci.nodes[4] = tmp_nodes[4],
+	    ci.nodes[5] = tmp_nodes[5];
+	}
+	else 
+	  continue;
       }
-      ci.set_pgt();
     }
+    nb_cv = cvlst.size();
     if (cvlst.size()) {
       std::sort(cvlst.begin(), cvlst.end());
       unsigned N = cvlst.front().pgt->dim();
       for (size_type cv=0; cv < nb_cv; ++cv) {
 	bool cvok = false;
 	gmsh_cv_info &ci = cvlst[cv];
-	//cout << "importing cv dim=" << int(ci.pgt->dim()) << " N=" << N << " region: " << ci.region << "\n";
+	//cout << "importing cv dim=" << int(ci.pgt->dim()) << " N=" << N << "
+	//region: " << ci.region << "\n";
 	if (ci.pgt->dim() == N) {
 	  size_type ic = m.add_convex(ci.pgt, ci.nodes.begin()); cvok = true;
 	  m.region(ci.region).add(ic);
@@ -127,8 +245,10 @@ namespace getfem {
 	    m.convex_to_point(ci.nodes[0]);
 	  for (bgeot::mesh_structure::ind_cv_ct::const_iterator 
 		 it = ct.begin(); it != ct.end(); ++it) {
-	    for (unsigned face=0; face < m.structure_of_convex(*it)->nb_faces(); ++face) {
-	      if (m.is_convex_face_having_points(*it,face,ci.nodes.size(),ci.nodes.begin())) {
+	    for (unsigned face=0;
+		 face < m.structure_of_convex(*it)->nb_faces(); ++face) {
+	      if (m.is_convex_face_having_points(*it,face,ci.nodes.size(),
+						 ci.nodes.begin())) {
 		m.region(ci.region+1000).add(*it,face);
 		cvok = true;
 	      }
@@ -138,7 +258,7 @@ namespace getfem {
 	  //else cout << "face found!\n";
 	}
 	if (!cvok)
-	  DAL_WARNING2("gmsh import ignored a convex of type "
+	  GMM_WARNING2("gmsh import ignored a convex of type "
 		       << bgeot::name_of_geometric_trans(ci.pgt));
       }
     }
@@ -147,7 +267,7 @@ namespace getfem {
 
   /* mesh file from GiD [http://gid.cimne.upc.es/]
 
-    supports linear and quadratic elements (quadrilaterals, use 9(or 27)-noded elements)
+  supports linear and quadratic elements (quadrilaterals, use 9(or 27)-noded elements)
   */
   static void import_gid_msh_file(std::ifstream& f, mesh& m) {
     /* read the node list */
@@ -159,21 +279,21 @@ namespace getfem {
     bool nodes_done = false;
     do {
       if (!f.eof()) f >> std::ws;
-      if (f.eof() || !ftool::read_until(f, "MESH")) break;
+      if (f.eof() || !bgeot::read_until(f, "MESH")) break;
       std::string selemtype;
-      f >> ftool::skip("DIMENSION") >> dim 
-	>> ftool::skip("ELEMTYPE") >> std::ws 
+      f >> bgeot::skip("DIMENSION") >> dim 
+	>> bgeot::skip("ELEMTYPE") >> std::ws 
 	>> selemtype 
-	>> ftool::skip("NNODE") >> nnode;
-      if (ftool::casecmp(selemtype, "linear")==0) { eltype = LIN;  }
-      else if (ftool::casecmp(selemtype, "triangle")==0) { eltype = TRI; }
-      else if (ftool::casecmp(selemtype, "quadrilateral")==0) { eltype = QUAD; }
-      else if (ftool::casecmp(selemtype, "tetrahedra")==0) { eltype = TETR; }
-      else if (ftool::casecmp(selemtype, "prisma")==0) { eltype = PRISM; }
-      else if (ftool::casecmp(selemtype, "hexahedra")==0) { eltype = HEX; }
-      else DAL_THROW(dal::failure_error, "unknown element type '"<< selemtype << "'");
+	>> bgeot::skip("NNODE") >> nnode;
+      if (bgeot::casecmp(selemtype, "linear")==0) { eltype = LIN;  }
+      else if (bgeot::casecmp(selemtype, "triangle")==0) { eltype = TRI; }
+      else if (bgeot::casecmp(selemtype, "quadrilateral")==0) { eltype = QUAD; }
+      else if (bgeot::casecmp(selemtype, "tetrahedra")==0) { eltype = TETR; }
+      else if (bgeot::casecmp(selemtype, "prisma")==0) { eltype = PRISM; }
+      else if (bgeot::casecmp(selemtype, "hexahedra")==0) { eltype = HEX; }
+      else GMM_ASSERT1(false, "unknown element type '"<< selemtype << "'");
       assert(!f.eof());
-      f >> ftool::skip("COORDINATES");
+      f >> bgeot::skip("COORDINATES");
       if (!nodes_done) {
 	dal::dynamic_array<base_node> gid_nodes;
 	dal::bit_vector gid_nodes_used;
@@ -182,7 +302,7 @@ namespace getfem {
 	  std::string ls;
 	  f >> std::ws;
 	  std::getline(f,ls);
-	  if (ftool::casecmp(ls, "END COORDINATES", 15)==0) break;
+	  if (bgeot::casecmp(ls, "END COORDINATES", 15)==0) break;
 	  std::stringstream s; s << ls; 
 	  size_type id;
 	  s >> id;
@@ -192,9 +312,7 @@ namespace getfem {
 	  //cerr << "ppoint " << id << ", " << n << endl;
 	} while (true);
 	
-	if (gid_nodes_used.card() == 0) {
-	  DAL_THROW(dal::failure_error,"no nodes in the mesh!");
-	}
+	GMM_ASSERT1(gid_nodes_used.card() != 0, "no nodes in the mesh!");
 	
 	/* suppression of unused dimensions */
 	std::vector<bool> direction_useless(3,true);
@@ -214,7 +332,7 @@ namespace getfem {
         }
       }
       
-      ftool::read_until(f, "ELEMENTS");
+      bgeot::read_until(f, "ELEMENTS");
       bgeot::pgeometric_trans pgt = NULL;
       std::vector<size_type> order(nnode); // ordre de GiD cf http://gid.cimne.upc.es/support/gid_11.subst#SEC160
       for (size_type i=0; i < nnode; ++i) order[i]=i;
@@ -264,17 +382,15 @@ namespace getfem {
 	  std::copy(lorder,lorder+nnode,order.begin());
 	}
       } break;
-      default: {
-	DAL_INTERNAL_ERROR("");
-      } break;
+      default: GMM_ASSERT1(false, ""); break;
       }
-      if (pgt == NULL) DAL_THROW(dal::failure_error, "unknown element type " << selemtype 
-				 << " with " << nnode << "nodes");
+      GMM_ASSERT1(pgt != NULL, "unknown element type " << selemtype 
+		  << " with " << nnode << "nodes");
       do {
 	std::string ls;
 	f >> std::ws;
 	std::getline(f,ls);
-	if (ftool::casecmp(ls, "END ELEMENTS", 12)==0) break;
+	if (bgeot::casecmp(ls, "END ELEMENTS", 12)==0) break;
 	std::stringstream s; s << ls; 
 	size_type cv_id;
 	s >> cv_id;
@@ -283,8 +399,7 @@ namespace getfem {
 	  size_type j;
 	  s >> j;
 	  cv_nodes[i] = msh_node_2_getfem_node.search(j);
-	  if (cv_nodes[i] == size_type(-1)) 
-	    DAL_THROW(failure_error, "Invalid node ID " << j 
+	  GMM_ASSERT1(cv_nodes[i] != size_type(-1), "Invalid node ID " << j 
 		      << " in GiD mesh convex num " << cv_id);
 	}
 	getfem_cv_nodes.resize(nnode);
@@ -315,7 +430,7 @@ namespace getfem {
 
   /* mesh file from emc2 [http://pauillac.inria.fr/cdrom/prog/unix/emc2/eng.htm], am_fmt format
 
-    (only triangular 2D meshes)
+  (only triangular 2D meshes)
   */
   static void import_am_fmt_file(std::ifstream& f, mesh& m) {
     /* read the node list */
@@ -323,7 +438,7 @@ namespace getfem {
     size_type nbs,nbt;
     base_node P(2);
     
-    f >> nbs >> nbt; ftool::read_until(f,"\n");
+    f >> nbs >> nbt; bgeot::read_until(f,"\n");
     tri.resize(nbt*3);
     for (size_type i=0; i < nbt*3; ++i) f >> tri[i];
     for (size_type j=0; j < nbs; ++j) {
@@ -331,7 +446,8 @@ namespace getfem {
       cerr.precision(16);
       P[0]=round_to_nth_significant_number(P[0],6); // force 9.999999E-1 to be converted to 1.0
       P[1]=round_to_nth_significant_number(P[1],6);
-      if (m.add_point(P) != j) DAL_INTERNAL_ERROR("ouch");
+      size_type jj = m.add_point(P);
+      GMM_ASSERT1(jj == j, "ouch");
     }
     for (size_type i=0; i < nbt*3; i+=3)
       m.add_triangle(tri[i]-1,tri[i+1]-1,tri[i+2]-1);
@@ -339,18 +455,19 @@ namespace getfem {
 
   /* mesh file from emc2 [http://pauillac.inria.fr/cdrom/prog/unix/emc2/eng.htm], am_fmt format
 
-    triangular/quadrangular 2D meshes
+  triangular/quadrangular 2D meshes
   */
   static void import_emc2_mesh_file(std::ifstream& f, mesh& m) {
     /* read the node list */
     std::vector<size_type> tri;      
     size_type nbs=0,nbt=0,nbq=0,dummy;
     base_node P(2);
-    ftool::read_until(f,"Vertices");
+    bgeot::read_until(f,"Vertices");
     f >> nbs;
     for (size_type j=0; j < nbs; ++j) {
       f >> P[0] >> P[1] >> dummy; 
-      if (m.add_point(P) != j) DAL_INTERNAL_ERROR("ouch");
+      size_type jj = m.add_point(P);
+      GMM_ASSERT1(jj == j, "ouch");
     }
     while (!f.eof()) {
       size_type ip[4];
@@ -377,46 +494,50 @@ namespace getfem {
     m.clear();
     try {
 
-      if (ftool::casecmp(format,"structured")==0)
+      if (bgeot::casecmp(format,"structured")==0)
 	{ regular_mesh(m, filename); return; }
       
       std::ifstream f(filename.c_str());
-      if (!f.good()) DAL_THROW(failure_error, "can't open file " << filename);
+      GMM_ASSERT1(f.good(), "can't open file " << filename);
       /* throw exceptions when an error occurs */
       f.exceptions(std::ifstream::badbit | std::ifstream::failbit);
       import_mesh(f,format,m);
       f.close();
     }
-    catch (dal::failure_error& exc) {
+    catch (failure_error& exc) {
       m.clear();
       throw exc;
     }
     catch (std::ios_base::failure& exc) {
       m.clear();
-      DAL_THROW(dal::failure_error, "error while importing " << format
-		<< " mesh file \"" << filename << "\" : " << exc.what());
+      GMM_ASSERT1(false, "error while importing " << format
+		  << " mesh file \"" << filename << "\" : " << exc.what());
     }
   }
 
   void import_mesh(std::ifstream& f, const std::string& format,
 		   mesh& m) {
-    if (ftool::casecmp(format,"gmsh")==0)
+    if (bgeot::casecmp(format,"gmsh")==0)
       import_gmsh_msh_file(f,m);
-    else if (ftool::casecmp(format,"gid")==0)
+    else if (bgeot::casecmp(format,"gmshv2")==0)
+      import_gmsh_msh_file(f,m,2);
+    else if (bgeot::casecmp(format,"gid")==0)
       import_gid_msh_file(f,m);
-    else if (ftool::casecmp(format,"am_fmt")==0)
+    else if (bgeot::casecmp(format,"am_fmt")==0)
       import_am_fmt_file(f,m);
-    else if (ftool::casecmp(format,"emc2_mesh")==0)
+    else if (bgeot::casecmp(format,"emc2_mesh")==0)
       import_emc2_mesh_file(f,m);
-    else DAL_THROW(dal::failure_error, "cannot import "
-		   << format << " mesh type : unknown mesh type");
+    else GMM_ASSERT1(false, "cannot import "
+		     << format << " mesh type : unknown mesh type");
   }
-
+  
   void import_mesh(const std::string& filename, mesh& msh) {
     if (filename.compare(0,4,"gid:")==0)
       getfem::import_mesh(filename.substr(4), "gid", msh);
     else if (filename.compare(0,5,"gmsh:") == 0) 
       getfem::import_mesh(filename.substr(5), "gmsh", msh);
+    else if (filename.compare(0,7,"gmshv2:") == 0) 
+      getfem::import_mesh(filename.substr(7), "gmshv2", msh);
     else if (filename.compare(0,7,"am_fmt:") == 0) 
       getfem::import_mesh(filename.substr(7), "am_fmt", msh);
     else if (filename.compare(0,10,"emc2_mesh:") == 0)
