@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2000-2006 Yves Renard
+// Copyright (C) 2000-2007 Yves Renard
 //
 // This file is a part of GETFEM++
 //
@@ -20,17 +20,17 @@
 //
 //========================================================================
 
-#include <dal_singleton.h>
-#include <getfem_mat_elem_type.h>
-#include <dal_tree_sorted.h>
-#include <getfem_mat_elem.h> /* for mat_elem_forget_mat_elem_type */
+#include "getfem/dal_singleton.h"
+#include "getfem/getfem_mat_elem_type.h"
+#include "getfem/dal_tree_sorted.h"
+#include "getfem/getfem_mat_elem.h" /* for mat_elem_forget_mat_elem_type */
 namespace getfem {
 
   bool operator < (const constituant &m, const constituant &n) {
     if (m.t < n.t) return true; if (m.t > n.t) return false;
     if (m.t == GETFEM_NONLINEAR_) {
       if (m.nlt < n.nlt) return true; if (n.nlt < m.nlt) return false;
-      if (m.nl_part < n.nl_part) return true; if (m.nl_part > n.nl_part) return false;
+      if (m.nl_part < n.nl_part) return true; if (m.nl_part > n.nl_part) return false; 
     }
     if (m.pfi < n.pfi) return true;
     return false;
@@ -42,7 +42,7 @@ namespace getfem {
     virtual bool compare(const static_stored_object_key &oo) const {
       const mat_elem_type_key &o
 	= dynamic_cast<const mat_elem_type_key &>(oo);
-      if (dal::lexicographical_less<mat_elem_type>()(*pmet, *(o.pmet)) < 0)
+      if (gmm::lexicographical_less<mat_elem_type>()(*pmet, *(o.pmet)) < 0)
 	return true;
       return false;
     }
@@ -138,13 +138,13 @@ namespace getfem {
     if (nl_part) {
       f.get_mi().resize(1); f.get_mi()[0] = 1;
     } else f.get_mi() = nlt->sizes();
-    return add_to_met_tab(f);
+    pmat_elem_type ret = add_to_met_tab(f);
+    return ret;
   }
 
   pmat_elem_type mat_elem_nonlinear(pnonlinear_elem_term nlt,
 				    std::vector<pfem> pfi) {
-    if (pfi.size() == 0)
-      DAL_THROW(dal::dimension_error, "mat_elem_nonlinear with no pfem!");
+    GMM_ASSERT1(pfi.size() != 0, "mat_elem_nonlinear with no pfem!");
     pmat_elem_type me = mat_elem_nonlinear_(nlt, pfi[0], 0);
     for (size_type i=1; i < pfi.size(); ++i)
       me = mat_elem_product(mat_elem_nonlinear_(nlt, pfi[i], i),me);
