@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 1999-2006 Yves Renard
+// Copyright (C) 1999-2007 Yves Renard
 //
 // This file is a part of GETFEM++
 //
@@ -20,9 +20,10 @@
 //
 //========================================================================
 
-#include <dal_singleton.h>
-#include <gmm_condition_number.h>
-#include <getfem_mesh.h>
+#include "getfem/dal_singleton.h"
+#include "gmm/gmm_condition_number.h"
+#include "getfem/getfem_mesh.h"
+#include "getfem/getfem_integration.h"
 
 namespace getfem {
 
@@ -71,7 +72,7 @@ namespace getfem {
 		if (found) continue;
 
 		base_node pt, barycentre
-		  =dal::mean_value(pgtsub->convex_ref()->points_of_face(fsub));
+		  =gmm::mean_value(pgtsub->convex_ref()->points_of_face(fsub));
 		pt = pgtsub->transform(barycentre, points_of_convex(icv[jc]));
 	
 		giv.init(points_of_convex(ic), pgt);
@@ -96,7 +97,7 @@ namespace getfem {
 	       ++fsub)
 	    if (r[icv[jc]][fsub+1]) {
 	      base_node pt, barycentre
-		= dal::mean_value(pgtsub->convex_ref()->points_of_face(fsub));
+		= gmm::mean_value(pgtsub->convex_ref()->points_of_face(fsub));
 	      pt = pgtsub->transform(barycentre, points_of_convex(icv[jc]));
 	      
 	      giv.init(points_of_convex(ic), pgt);
@@ -180,18 +181,34 @@ namespace getfem {
   }
 #endif
 
+  size_type mesh::add_point(const base_node &pt) {
+    static scalar_type prec_factor = gmm::default_tol(scalar_type())
+      * scalar_type(10000);
+    scalar_type npt = gmm::vect_norm2(pt);
+    if (nb_points() == 0) {
+      dimension = pt.size();
+      characteristic_size = max_radius = npt;
+      pts.comparator().c.eps = characteristic_size * prec_factor;
+      return pts.add(pt);
+    }
+    else {
+      GMM_ASSERT1(pt.size() == dimension,
+		  "mesh::add_point : dimensions mismatch");
+      max_radius = std::max(max_radius, npt);
 
-  size_type mesh::add_point(const base_node &pt, bool norepeat) {
-    if (dimension == dim_type(-1)) dimension = pt.size();
-    if (pt.size() != dimension)
-      DAL_THROW(dimension_error, "mesh::add_point : dimensions mismatch");
-    
-    if (norepeat) {
-      bool present;
-      size_type i = pts.add_norepeat(pt, false, &present);
+      if (max_radius > scalar_type(10)*characteristic_size) {
+	characteristic_size = max_radius;
+	pts.comparator().c.eps = characteristic_size * prec_factor;
+	pts.resort();
+      }
+      
+      bgeot::basic_mesh::PT_TAB::const_sorted_iterator it = pts.sorted_ge(pt); 
+      if (it.index() != ST_NIL && pts.comparator()(pt, *it) == 0)
+	{ return it.index(); }
+      
+      size_type i = pts.add(pt);
       return i;
     }
-    else return pts.add(pt);
   }
 
   void mesh::optimize_structure() {
@@ -225,8 +242,7 @@ namespace getfem {
 
   void mesh::transformation(base_matrix M) {
     base_small_vector w(M.nrows());
-    if (gmm::mat_nrows(M) == 0 || gmm::mat_ncols(M) != dim()) 
-      DAL_THROW(dal::dimension_error,
+    GMM_ASSERT1(gmm::mat_nrows(M) != 0 && gmm::mat_ncols(M) == dim(),
 		"invalid dimensions for the transformation matrix");
     for (dal::bv_visitor i(pts.index()); !i.finished(); ++i) {
       w = pts[i];
@@ -357,6 +373,14 @@ namespace getfem {
     return bgeot::compute_local_basis(c, f);
   }
 
+  scalar_type  mesh::convex_area_estimate(size_type ic, size_type deg) const { 
+    base_matrix G;
+    bgeot::vectors_to_base_matrix(G, points_of_convex(ic));
+    papprox_integration pai = 
+      classical_approx_im(trans_of_convex(ic), deg)->approx_method();
+    return getfem::convex_area_estimate(trans_of_convex(ic), G, pai);
+  }
+
   scalar_type  mesh::convex_quality_estimate(size_type ic) const { 
     base_matrix G;
     bgeot::vectors_to_base_matrix(G, points_of_convex(ic));
@@ -396,7 +420,10 @@ namespace getfem {
   };
 
   void mesh::read_from_file(std::istream &ist) {
-   
+    /* if the application does a setlocale, then we have a problem..*/
+    GMM_ASSERT1(atof("1.5") == 1.5f,
+	       "Go fix your locales! The numerical separator MUST be the dot");
+
     dal::bit_vector npt;
     dal::dynamic_array<double> tmpv;
     std::string tmp;
@@ -405,44 +432,40 @@ namespace getfem {
     ist.precision(16);
     clear();
     ist.seekg(0);ist.clear();
-    ftool::read_until(ist, "BEGIN POINTS LIST");
+    bgeot::read_until(ist, "BEGIN POINTS LIST");
 
     while (!te) {
-      if (please_get) ftool::get_token(ist, tmp); else please_get = true;
+      if (please_get) bgeot::get_token(ist, tmp); else please_get = true;
 
-      if (!ftool::casecmp(tmp, "END"))
+      if (!bgeot::casecmp(tmp, "END"))
       { te = true; }
-      else if (!ftool::casecmp(tmp, "POINT")) {
-	ftool::get_token(ist, tmp);
+      else if (!bgeot::casecmp(tmp, "POINT")) {
+	bgeot::get_token(ist, tmp);
         size_type ip = atoi(tmp.c_str());
         dim_type d = 0;
-	if (npt.is_in(ip))
-	  DAL_THROW(failure_error, 
+	GMM_ASSERT1(!npt.is_in(ip), 
 		    "Two points with the same index. loading aborted.");
 	npt.add(ip);
-	ftool::get_token(ist, tmp);
+	bgeot::get_token(ist, tmp);
 	while (isdigit(tmp[0]) || tmp[0] == '-' || tmp[0] == '+'
 	                       || tmp[0] == '.')
-	  { tmpv[d++] = atof(tmp.c_str()); ftool::get_token(ist, tmp); }
+	  { tmpv[d++] = atof(tmp.c_str()); bgeot::get_token(ist, tmp); }
 	please_get = false;
 	if (dimension == dim_type(-1)) dimension = d;
-	else if (dimension != d)
-	  DAL_THROW(failure_error, "Points of different dimensions.");
+	else GMM_ASSERT1(dimension == d, "Points of different dimensions.");
 	base_node v(d);
 	for (size_type i = 0; i < d; i++) v[i] = tmpv[i];
 	size_type ipl = add_point(v);
 	if (ip != ipl) {
-	  if (npt.is_in(ipl))
-	    DAL_THROW(failure_error, 
-		      "Two points [#" << ip << " and #" << ipl
-		      << "] with the same coords. loading aborted.");
+	  GMM_ASSERT1(!npt.is_in(ipl), "Two points [#" << ip << " and #" << ipl
+		      << "] with the same coords "<< v <<". loading aborted.");
 	  swap_points(ip, ipl);
 	}
       } else if (tmp.size()) {
-	DAL_THROW(failure_error, "Syntax error in file, at token '" << tmp
-		  << "', pos=" << std::streamoff(ist.tellg()));
+	GMM_ASSERT1(false, "Syntax error in file, at token '" << tmp
+		    << "', pos=" << std::streamoff(ist.tellg()));
       } else if (ist.eof()) {
-	DAL_THROW(failure_error, "Unexpected end of stream");	
+	GMM_ASSERT1(false, "Unexpected end of stream while reading mesh");
       }
     }
 
@@ -451,23 +474,22 @@ namespace getfem {
     dal::bit_vector ncv;
     
     ist.seekg(0);
-    if (!ftool::read_until(ist, "BEGIN MESH STRUCTURE DESCRIPTION"))
-      DAL_THROW(failure_error, "This seems not to be a mesh file");
+    if (!bgeot::read_until(ist, "BEGIN MESH STRUCTURE DESCRIPTION"))
+      GMM_ASSERT1(false, "This seems not to be a mesh file");
 
     while (!tend) {
-      tend = !ftool::get_token(ist, tmp);
-      if (!ftool::casecmp(tmp, "END"))
+      tend = !bgeot::get_token(ist, tmp);
+      if (!bgeot::casecmp(tmp, "END"))
       { tend = true; }
-      else if (!ftool::casecmp(tmp, "CONVEX")) {
+      else if (!bgeot::casecmp(tmp, "CONVEX")) {
         size_type ic;
-	ftool::get_token(ist, tmp);
+	bgeot::get_token(ist, tmp);
         ic = gmm::abs(atoi(tmp.c_str()));
-	if (ncv.is_in(ic)) 
-	  DAL_THROW(failure_error,
+	GMM_ASSERT1(!ncv.is_in(ic),
 		    "Negative or repeated index, loading aborted.");
 	ncv.add(ic);
 
-	int rgt = ftool::get_token(ist, tmp);
+	int rgt = bgeot::get_token(ist, tmp);
 	if (rgt != 3) { // for backward compatibility with version 1.7
 	  char c; ist.get(c);
 	  while (!isspace(c)) { tmp.push_back(c); ist.get(c); }
@@ -479,20 +501,20 @@ namespace getfem {
 	cv[ic].cstruct = pgt;
 	cv[ic].pts.resize(nb);
 	for (size_type i = 0; i < nb; i++) {
-	  ftool::get_token(ist, tmp);	  
+	  bgeot::get_token(ist, tmp);	  
 	  cv[ic].pts[i] = gmm::abs(atoi(tmp.c_str()));
 	}
       }
       else if (tmp.size()) {
-	DAL_THROW(failure_error, "Syntax error reading a mesh file "
-		  " at pos " << std::streamoff(ist.tellg())
-		  << "(expecting 'CONVEX' or 'END', found '" << tmp << "')"); 
+	GMM_ASSERT1(false, "Syntax error reading a mesh file "
+		    " at pos " << std::streamoff(ist.tellg())
+		    << "(expecting 'CONVEX' or 'END', found '"<< tmp << "')"); 
       } else if (ist.eof()) {
-	DAL_THROW(failure_error, "Unexpected end of stream "
-		  << "(missing BEGIN MESH/END MESH ?)");
+	GMM_ASSERT1(false, "Unexpected end of stream "
+		    << "(missing BEGIN MESH/END MESH ?)");
       }
     }
-    ist >> ftool::skip("MESH STRUCTURE DESCRIPTION");
+    ist >> bgeot::skip("MESH STRUCTURE DESCRIPTION");
 
     for (dal::bv_visitor ic(ncv); !ic.finished(); ++ic) {
       size_type i = add_convex(cv[ic].cstruct, cv[ic].pts.begin());
@@ -501,36 +523,36 @@ namespace getfem {
 
     tend = false;
     while (!tend) {
-      tend = !ftool::get_token(ist, tmp);
+      tend = !bgeot::get_token(ist, tmp);
       // bool error = false;
-      if (ftool::casecmp(tmp, "BEGIN")==0) {
-	ftool::get_token(ist, tmp);
-	if (ftool::casecmp(tmp, "BOUNDARY")==0 ||
-	    ftool::casecmp(tmp, "REGION")==0) {
-	  ftool::get_token(ist, tmp);
+      if (bgeot::casecmp(tmp, "BEGIN")==0) {
+	bgeot::get_token(ist, tmp);
+	if (bgeot::casecmp(tmp, "BOUNDARY")==0 ||
+	    bgeot::casecmp(tmp, "REGION")==0) {
+	  bgeot::get_token(ist, tmp);
 	  size_type bnum = atoi(tmp.c_str());
-	  ftool::get_token(ist, tmp);
+	  bgeot::get_token(ist, tmp);
 	  while (true) {
-	    if (ftool::casecmp(tmp, "END")!=0) {
+	    if (bgeot::casecmp(tmp, "END")!=0) {
 	      size_type ic = atoi(tmp.c_str());
-	      ftool::get_token(ist, tmp);
+	      bgeot::get_token(ist, tmp);
 	      if (tmp[0] == '/') {
-		ftool::get_token(ist, tmp);
-		if (!ftool::casecmp(tmp, "END")) break;
+		bgeot::get_token(ist, tmp);
+		if (!bgeot::casecmp(tmp, "END")) break;
 		size_type f = atoi(tmp.c_str());
 		region(bnum).add(ic, f);
-		ftool::get_token(ist, tmp);
+		bgeot::get_token(ist, tmp);
 	      }
 	      else {
 		region(bnum).add(ic);
-		if (!ftool::casecmp(tmp, "END")) break;
+		if (!bgeot::casecmp(tmp, "END")) break;
 	      }
 	    } else break;
 	  }
-	  ftool::get_token(ist, tmp);
-	  ftool::get_token(ist, tmp);
+	  bgeot::get_token(ist, tmp);
+	  bgeot::get_token(ist, tmp);
 	} else tend = true;
-	/*else DAL_THROW(failure_error, "Syntax error in file at token '"
+	/*else GMM_ASSERT1(false, "Syntax error in file at token '"
 	  << tmp << "' [pos=" << std::streamoff(ist.tellg())
 	  << "]");*/
       } else tend=true;
@@ -539,8 +561,7 @@ namespace getfem {
 
   void mesh::read_from_file(const std::string &name) { 
     std::ifstream o(name.c_str());
-    if (!o) DAL_THROW(file_not_found_error,
-		      "Mesh file '" << name << "' does not exist");
+    GMM_ASSERT1(o, "Mesh file '" << name << "' does not exist");
     read_from_file(o); 
     o.close();
   }
@@ -615,8 +636,7 @@ namespace getfem {
 
   void mesh::write_to_file(const std::string &name) const {
     std::ofstream o(name.c_str());
-    if (!o)
-      DAL_THROW(failure_error, "impossible to write to file '" << name << "'");
+    GMM_ASSERT1(o, "impossible to write to file '" << name << "'");
     o << "% GETFEM MESH FILE " << '\n';
     o << "% GETFEM VERSION " << GETFEM_VERSION << '\n' << '\n' << '\n';
     write_to_file(o);
@@ -649,6 +669,28 @@ namespace getfem {
     return pbm[N-1];
   }
     
+  
+  scalar_type convex_area_estimate(bgeot::pgeometric_trans pgt,
+				   const base_matrix& G, 
+				   const approx_integration *pai) {
+    double area(0);
+    static bgeot::pgeometric_trans pgt_old = 0;
+    static papprox_integration pai_old = 0;
+    static bgeot::pgeotrans_precomp pgp = 0;
+    static pintegration_method pim = 0;
+    if (pgt_old != pgt || pai_old != pai) {
+      pgt_old = pgt;
+      pgp = bgeot::geotrans_precomp
+	(pgt,&pai->integration_points());
+    }
+    bgeot::geotrans_interpolation_context gic(pgp, 0, G);
+    for (size_type i = 0; i < pai->nb_points_on_convex(); ++i) {
+      gic.set_ii(i);
+      area += pai->coeff(i) * gic.J();
+    }
+    return area;
+  }
+
   /* TODO : use the geotrans from an "equilateral" reference element to
      the real element 
      check if the sign of the determinants does change
@@ -726,7 +768,8 @@ namespace getfem {
       if (m.structure_of_convex(i.cv())->dim() == m.dim()) {
 	for (size_type f = 0; f < m.structure_of_convex(i.cv())->nb_faces();
 	     f++) {
-	  if (!m.is_convex_having_neighbour(i.cv(),f)) {
+	  size_type cv2 = m.neighbour_of_convex(i.cv(), f);
+	  if (cv2 == size_type(-1) || !cvlst.is_in(cv2)) {
 	    flist.add(i.cv(),f);
 	  }
 	}
@@ -740,8 +783,8 @@ namespace getfem {
     base_node pt(dim+1);
     out.clear();
     size_type nbpt = in.points().index().last()+1;
-    if (nbpt != in.points().index().card()) 
-      DAL_THROW(dal::failure_error, "please optimize the mesh before using "
+    GMM_ASSERT1(nbpt == in.points().index().card(),
+		"please optimize the mesh before using "
 		"it as a base for prismatic mesh");
     for (size_type i = 0; i < nbpt; ++i) {
       std::copy(in.points()[i].begin(), in.points()[i].end(),pt.begin());
@@ -923,9 +966,8 @@ namespace getfem {
 
   void mesh::Bank_refine_normal_convex(size_type i) {
     bgeot::pgeometric_trans pgt = trans_of_convex(i);
-    if (pgt->basic_structure() != bgeot::simplex_structure(pgt->dim()))
-      DAL_THROW(failure_error, "Sorry, refinement is only working "
-		"with simplices.");
+    GMM_ASSERT1(pgt->basic_structure() == bgeot::simplex_structure(pgt->dim()),
+		"Sorry, refinement is only working with simplices.");
     
     const std::vector<size_type> &loc_ind = pgt->vertices();
     for (size_type ip1 = 0; ip1 < loc_ind.size(); ++ip1)
