@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 1999-2006 Yves Renard
+// Copyright (C) 1999-2007 Yves Renard
 //
 // This file is a part of GETFEM++
 //
@@ -22,12 +22,12 @@
 
 
 #include <queue>
-#include <dal_singleton.h>
-#include <getfem_mesh_fem.h>
+#include "getfem/dal_singleton.h"
+#include "getfem/getfem_mesh_fem.h"
 
 namespace getfem {
   
-  dal::bit_vector mesh_fem::dof_on_set(const mesh_region &b) const {
+  dal::bit_vector mesh_fem::dof_on_region(const mesh_region &b) const {
     if (!dof_enumeration_made) this->enumerate_dof();
     dal::bit_vector res;
     for (getfem::mr_visitor v(b,linked_mesh()); !v.finished(); ++v) {
@@ -73,6 +73,8 @@ namespace getfem {
     fe_convex.swap(m.icv1, m.icv2);
     f_elems.swap(m.icv1, m.icv2);
   }
+
+  /* keep the FEM and dof_partition when the mesh is refined */
   void mesh_fem::receipt(const MESH_REFINE_CONVEX &m) { 
     if (m.is_refine) {
       if (fe_convex[m.icv])
@@ -80,10 +82,14 @@ namespace getfem {
 	  f_elems[m.sub_cv_list[i]] = f_elems[m.icv];
 	  fe_convex.add(m.sub_cv_list[i]);
 	}
+      for (size_type i = 0; i < m.sub_cv_list.size(); ++i) {
+	set_dof_partition(m.sub_cv_list[i], get_dof_partition(m.icv));
+      }
     }
     else if (fe_convex[m.sub_cv_list[0]]) {
       f_elems[m.icv] = f_elems[m.sub_cv_list[0]];
       fe_convex.add(m.icv);
+      set_dof_partition(m.icv, get_dof_partition(m.sub_cv_list[0]));
     }
   }
    
@@ -96,16 +102,15 @@ namespace getfem {
       }
     }
     else {
-      if (linked_mesh_->structure_of_convex(cv)->basic_structure() 
-	  != pf->basic_structure(cv))
-	DAL_THROW(dal::failure_error,
+      GMM_ASSERT1(linked_mesh_->structure_of_convex(cv)->basic_structure() 
+		  == pf->basic_structure(cv),
 		  "Incompatibility between fem " << name_of_fem(pf) << 
 		  " and mesh element " <<
 		  name_of_geometric_trans(linked_mesh_->trans_of_convex(cv)));
-      if ((Qdim % pf->target_dim()) != 0 && pf->target_dim() != 1)
-	DAL_THROW(dal::failure_error,
-		  "Incompatibility between Qdim=" << int(Qdim) << " and target_dim " <<
-		  int(pf->target_dim()) << " of " << name_of_fem(pf));
+      GMM_ASSERT1((Qdim % pf->target_dim()) == 0 || pf->target_dim() == 1,
+		  "Incompatibility between Qdim=" << int(Qdim) <<
+		  " and target_dim " << int(pf->target_dim()) << " of " <<
+		  name_of_fem(pf));
       if (!fe_convex.is_in(cv) || f_elems[cv] != pf) {
 	fe_convex.add(cv);
 	f_elems[cv] = pf;
@@ -168,13 +173,13 @@ namespace getfem {
 	   linked_mesh().points_of_convex(cv));
       }
     }
-    DAL_THROW(failure_error, "Inexistent dof");
+    GMM_ASSERT1(false, "Inexistent dof");
   }
 
   dim_type mesh_fem::dof_qdim(size_type d) const {
     if (!dof_enumeration_made) enumerate_dof();
     size_type cv = first_convex_of_dof(d);
-    if (cv == size_type(-1)) DAL_THROW(failure_error, "Inexistent dof");
+    GMM_ASSERT1(cv != size_type(-1), "Inexistent dof");
     size_type tdim = f_elems[cv]->target_dim();
     return dof_structure.ind_in_convex_of_point(cv, d) % (Qdim / tdim);
   }
@@ -202,16 +207,19 @@ namespace getfem {
       size_type j = dof_structure.first_convex_of_point(i);
       if (j != size_type(-1)) return dof_structure.convex_to_point(i);
     }
-    DAL_THROW(failure_error, "Inexistent dof");
+    GMM_ASSERT1(false, "Inexistent dof");
   }
 
   struct dof_comp_ { 
-    dal::approx_less<scalar_type> comp;
+    gmm::approx_less<scalar_type> comp;
     int operator()(const fem_dof& m, const fem_dof& n) const { 
-      int d = dal::lexicographical_compare(m.P.begin(), m.P.end(),
+      int d = gmm::lexicographical_compare(m.P.begin(), m.P.end(),
 					   n.P.begin(), n.P.end(), comp);
       if (d != 0) return (d < 0);
-      return dof_description_compare(m.pnd, n.pnd) < 0;
+      if (m.part == n.part)
+	return dof_description_compare(m.pnd, n.pnd) < 0;
+      else if (m.part < n.part) return true;
+      else /*if (m.part > n.part)*/ return false;
     }
     dof_comp_(double e = 1.0E-10) : comp(e) { }
   };
@@ -228,8 +236,7 @@ namespace getfem {
       return;
     }
     const std::vector<size_type> &cmk = linked_mesh().cuthill_mckee_ordering();
-    // double t = dal::uclock_sec();
-
+    
     dof_sort_type dof_sort;
     dal::bit_vector encountered_global_dof;
     dal::dynamic_array<size_type> ind_global_dof;
@@ -262,11 +269,12 @@ namespace getfem {
       for (size_type i = 0; i < nbd; i++) {
 	fd.P.resize(linked_mesh().dim()); 
 	fd.pnd = pf->dof_types()[i];
- 
+	fd.part = get_dof_partition(cv);
 	if (fd.pnd == andof) {
 	  size_type num = pf->index_of_global_dof(cv, i);
 	  if (!(encountered_global_dof[num])) {
 	    ind_global_dof[num] = nbdof;
+	    // cout << "Global dof " << num << " is numbered " << nbdof <<endl;
 	    nbdof += Qdim / pf->target_dim();
 	    encountered_global_dof[num] = true;
 	  }
@@ -277,7 +285,8 @@ namespace getfem {
 	} else {
 	  pgp->transform(linked_mesh().points_of_convex(cv), i, fd.P);
 
-	  std::pair<dof_sort_type::iterator, bool> pa = dof_sort.insert(std::make_pair(fd, nbdof));
+	  std::pair<dof_sort_type::iterator, bool>
+	    pa = dof_sort.insert(std::make_pair(fd, nbdof));
 	  if (pa.second) {
 	    tab[i] = nbdof;
 	    nbdof += Qdim / pf->target_dim();
@@ -290,15 +299,8 @@ namespace getfem {
     
     dof_enumeration_made = true;
     nb_total_dof = nbdof;
-    
-//    enumerate_dof_time += dal::uclock_sec() - t;
-//    cerr << "enumerate_dof_time: " << enumerate_dof_time << " sec [nbd=" << nbdof << "]\n";
   }
 
-
-
-
-
   void mesh_fem::clear(void) {
     fe_convex.clear();
     dof_enumeration_made = false;
@@ -312,10 +314,10 @@ namespace getfem {
     linked_mesh_ = &me;
     this->add_dependency(me);
     add_sender(me.lmsg_sender(), *this,
-	       lmsg::mask(MESH_CLEAR::ID) | lmsg::mask(MESH_SUP_CONVEX::ID) |
-	       lmsg::mask(MESH_SWAP_CONVEX::ID) | lmsg::mask(MESH_DELETE::ID) |
-	       lmsg::mask(MESH_ADD_CONVEX::ID)|
-	       lmsg::mask(MESH_REFINE_CONVEX::ID));
+	       mask(MESH_CLEAR::ID)   | mask(MESH_SUP_CONVEX::ID) |
+	       mask(MESH_SWAP_CONVEX::ID) | mask(MESH_DELETE::ID) |
+	       mask(MESH_ADD_CONVEX::ID)|
+	       mask(MESH_REFINE_CONVEX::ID));
   }
 
   mesh_fem::~mesh_fem() {}
@@ -328,41 +330,43 @@ namespace getfem {
     ist.precision(16);
     clear();
     ist.seekg(0);ist.clear();
-    ftool::read_until(ist, "BEGIN MESH_FEM");
+    bgeot::read_until(ist, "BEGIN MESH_FEM");
 
     while (true) {
-      ist >> std::ws; ftool::get_token(ist, tmp);
-      if (ftool::casecmp(tmp, "END")==0) {
+      ist >> std::ws; bgeot::get_token(ist, tmp);
+      if (bgeot::casecmp(tmp, "END")==0) {
 	break;
-      } else if (ftool::casecmp(tmp, "CONVEX")==0) {
-	ftool::get_token(ist, tmp);
+      } else if (bgeot::casecmp(tmp, "CONVEX")==0) {
+	bgeot::get_token(ist, tmp);
 	size_type ic = atoi(tmp.c_str());
-	if (!linked_mesh().convex_index().is_in(ic)) {
-	  DAL_THROW(failure_error, "Convex " << ic <<
+	GMM_ASSERT1(linked_mesh().convex_index().is_in(ic), "Convex " << ic <<
 		    " does not exist, are you sure "
 		    "that the mesh attached to this object is right one ?");
-	}
 	
-	int rgt = ftool::get_token(ist, tmp);
+	int rgt = bgeot::get_token(ist, tmp);
 	if (rgt != 3) { // for backward compatibility
 	  char c; ist.get(c);
 	  while (!isspace(c)) { tmp.push_back(c); ist.get(c); }
 	}
 	getfem::pfem fem = getfem::fem_descriptor(tmp);
-	if (!fem) DAL_THROW(failure_error, "could not create the FEM '" 
-			    << tmp << "'");
+	GMM_ASSERT1(fem, "could not create the FEM '" << tmp << "'");
 	set_finite_element(ic, fem);
-      } else if (ftool::casecmp(tmp, "BEGIN")==0) {
-	ftool::get_token(ist, tmp);
-	if (!ftool::casecmp(tmp, "DOF_ENUMERATION")) {
+      } else if (bgeot::casecmp(tmp, "BEGIN")==0) {
+	bgeot::get_token(ist, tmp);
+	if (bgeot::casecmp(tmp, "DOF_PARTITION") == 0) {
+	  for (dal::bv_visitor cv(convex_index()); !cv.finished(); ++cv) {
+	    size_type d; ist >> d; set_dof_partition(cv,d);
+	  }
+	  ist >> bgeot::skip("END DOF_PARTITION");
+	} else if (bgeot::casecmp(tmp, "DOF_ENUMERATION") == 0) {
 	  dal::bit_vector doflst;
 	  dof_structure.clear(); dof_enumeration_made = false; touch();
 	  while (true) {
-	    ftool::get_token(ist, tmp);
-	    if (ftool::casecmp(tmp, "END")==0) { 
+	    bgeot::get_token(ist, tmp);
+	    if (bgeot::casecmp(tmp, "END")==0) { 
 	      break;
 	    }
-	    ftool::get_token(ist, tmp2);
+	    bgeot::get_token(ist, tmp2);
 
 	    size_type ic = atoi(tmp.c_str());
 	    std::vector<size_type> tab;
@@ -377,50 +381,45 @@ namespace getfem {
 	      }
 	      dof_structure.add_convex_noverif
 		(fem_of_element(ic)->structure(ic), tab.begin(), ic);
-	    } else DAL_THROW(failure_error, "Missing convex or wrong number "
-			     << "in dof enumeration: '" 
-			     << tmp << "' [pos="
-			     << std::streamoff(ist.tellg())<<"]");
-	    /*ftool::get_token(ist, tmp);
+	    } else GMM_ASSERT1(false, "Missing convex or wrong number "
+			       << "in dof enumeration: '" 
+			       << tmp << "' [pos="
+			       << std::streamoff(ist.tellg())<<"]");
+	    /*bgeot::get_token(ist, tmp);
 	      cerr << " tok: '" << tmp << "'\n";*/
 	  } 
 	  dof_read = true;
 	  this->dof_enumeration_made = true;
 	  touch();
 	  this->nb_total_dof = doflst.card();
-	  ist >> ftool::skip("DOF_ENUMERATION");
+	  ist >> bgeot::skip("DOF_ENUMERATION");
 	} else if (tmp.size())
-	  DAL_THROW(failure_error, "Syntax error in file at token '"
-		    << tmp << "' [pos=" << std::streamoff(ist.tellg())
-							  << "]");
-      } else if (ftool::casecmp(tmp, "QDIM")==0) {
-	if (dof_read)
-	  DAL_THROW(failure_error, "Can't change QDIM after dof enumeration");
-	ftool::get_token(ist, tmp);
+	  GMM_ASSERT1(false, "Syntax error in file at token '"
+		      << tmp << "' [pos=" << std::streamoff(ist.tellg())
+		      << "]");
+      } else if (bgeot::casecmp(tmp, "QDIM")==0) {
+	GMM_ASSERT1(!dof_read, "Can't change QDIM after dof enumeration");
+	bgeot::get_token(ist, tmp);
 	int q = atoi(tmp.c_str());
-	if (q <= 0 || q > 250) DAL_THROW(failure_error, "invalid qdim: "<<q);
+	GMM_ASSERT1(q > 0 && q <= 250, "invalid qdim: " << q);
 	set_qdim(q);
       } else if (tmp.size()) {
-	DAL_THROW(failure_error, "Unexpected token '" << tmp <<
-		  "' [pos=" << std::streamoff(ist.tellg()) << "]");
+	GMM_ASSERT1(false, "Unexpected token '" << tmp <<
+		    "' [pos=" << std::streamoff(ist.tellg()) << "]");
       } else if (ist.eof()) {
-	DAL_THROW(failure_error, "Unexpected end of stream "
-		  << "(missing BEGIN MESH_FEM/END MESH_FEM ?)");	
+	GMM_ASSERT1(false, "Unexpected end of stream "
+		    << "(missing BEGIN MESH_FEM/END MESH_FEM ?)");	
       }
     }
   }
 
-  void mesh_fem::read_from_file(const std::string &name)
-  { 
+  void mesh_fem::read_from_file(const std::string &name) { 
     std::ifstream o(name.c_str());
-    if (!o) DAL_THROW(file_not_found_error,
-		      "Mesh_fem file '" << name << "' does not exist");
+    GMM_ASSERT1(o, "Mesh_fem file '" << name << "' does not exist");
     read_from_file(o);
-    o.close();
   }
 
-  void mesh_fem::write_to_file(std::ostream &ost) const
-  {
+  void mesh_fem::write_to_file(std::ostream &ost) const {
     ost << '\n' << "BEGIN MESH_FEM" << '\n' << '\n';
     ost << "QDIM " << size_type(get_qdim()) << '\n';
     for (dal::bv_visitor cv(convex_index()); !cv.finished(); ++cv) {
@@ -428,6 +427,15 @@ namespace getfem {
       ost << " \'" << name_of_fem(fem_of_element(cv));
       ost << "\'\n";
     }
+    if (!dof_partition.empty()) {
+      ost << " BEGIN DOF_PARTITION\n";
+      unsigned i = 0;
+      for (dal::bv_visitor cv(convex_index()); !cv.finished(); ++cv) {
+	ost << " " << get_dof_partition(cv); if ((++i % 20) == 0) ost << "\n";
+      }
+      ost << "\n";
+      ost << " END DOF_PARTITION\n";
+    }
 
     ost << " BEGIN DOF_ENUMERATION " << '\n';
     for (dal::bv_visitor cv(convex_index()); !cv.finished(); ++cv) {
@@ -446,16 +454,13 @@ namespace getfem {
     ost << "END MESH_FEM" << '\n';
   }
 
-  void mesh_fem::write_to_file(const std::string &name, bool with_mesh) const
-  {
+  void mesh_fem::write_to_file(const std::string &name, bool with_mesh) const {
     std::ofstream o(name.c_str());
-    if (!o)
-      DAL_THROW(failure_error, "impossible to open file '" << name << "'");
+    GMM_ASSERT1(o, "impossible to open file '" << name << "'");
     o << "% GETFEM MESH_FEM FILE " << '\n';
     o << "% GETFEM VERSION " << GETFEM_VERSION << '\n' << '\n' << '\n';
     if (with_mesh) linked_mesh().write_to_file(o);
     write_to_file(o);
-    o.close();
   }
 
   struct mf__key_ {
@@ -501,7 +506,7 @@ namespace getfem {
 	// the list of mesh pointers should be sorted ...
 	for (mesh_fem_tab::iterator itt = mfs.begin(); itt != mfs.end(); ++itt)
 	  if (itt->first.pmesh == &msh) goto nothing_to_do;
-	add_sender(msh.lmsg_sender(), *this, lmsg::mask(MESH_DELETE::ID));
+	add_sender(msh.lmsg_sender(), *this, mask(MESH_DELETE::ID));
 	
       nothing_to_do :
 	
