@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2005-2006 Julien Pommier
+// Copyright (C) 2005-2007 Julien Pommier
 //
 // This file is a part of GETFEM++
 //
@@ -20,10 +20,32 @@
 //
 //========================================================================
 
-#include <getfem_mesh_level_set.h>
+#include "getfem/getfem_mesh_level_set.h"
 
 
 namespace getfem {
+
+  std::ostream &operator<<(std::ostream &os, const mesh_level_set::zone &z) {
+    os << "zone[";
+    for (mesh_level_set::zone::const_iterator it = z.begin();
+	 it != z.end(); ++it) {
+      if (it != z.begin()) os << ", ";
+      os << **it; 
+    }
+    os << "]";
+    return os;
+  }
+
+  std::ostream &operator<<(std::ostream &os, const mesh_level_set::zoneset &zs) {
+    os << "zoneset[";
+    for (mesh_level_set::zoneset::const_iterator it = zs.begin(); 
+	 it != zs.end(); ++it) {
+      if (it != zs.begin()) os << "; ";
+      os << **it;
+    }
+    os << "]";
+    return os;
+  }
   
 
 #ifdef DEBUG_LS
@@ -83,8 +105,8 @@ struct Chrono {
     linked_mesh_ = &me; is_adapted_ = false; 
     this->add_dependency(me);
     add_sender(me.lmsg_sender(), *this,
-	       lmsg::mask(MESH_CLEAR::ID) | lmsg::mask(MESH_SUP_CONVEX::ID) |
-	       lmsg::mask(MESH_SWAP_CONVEX::ID) | lmsg::mask(MESH_DELETE::ID));
+	       mask(MESH_CLEAR::ID) | mask(MESH_SUP_CONVEX::ID) |
+	       mask(MESH_SWAP_CONVEX::ID) | mask(MESH_DELETE::ID));
   }
 
   mesh_level_set::~mesh_level_set() {}
@@ -122,7 +144,8 @@ struct Chrono {
 	  base_node &P = m.points()[ipts[i]];
 	  // if (cts.card() > 1)
 	  //   cout << "WARNING, projection sur " << cts << endl;
-	  pure_multi_constraint_projection(list_constraints, P, cts);
+	  if (!pure_multi_constraint_projection(list_constraints, P, cts))
+	    GMM_WARNING1("Pure multi has failed in interpolate_face !!");
 	  // dist(P, new_cts);
 	}
       }
@@ -133,12 +156,13 @@ struct Chrono {
   struct point_stock {
     
     dal::dynamic_tree_sorted
-    <base_node, dal::lexicographical_less<base_node,
-		dal::approx_less<scalar_type> > > points;
+    <base_node, gmm::lexicographical_less<base_node,
+		gmm::approx_less<scalar_type> > > points;
     std::vector<dal::bit_vector> constraints_;
     std::vector<scalar_type> radius_;
     const std::vector<const mesher_signed_distance*> &list_constraints;
-    
+    scalar_type radius_cv;
+
     void clear(void) { points.clear(); constraints_.clear();radius_.clear(); }
     scalar_type radius(size_type i) const { return radius_[i]; }
     const dal::bit_vector &constraints(size_type i) const
@@ -161,7 +185,8 @@ struct Chrono {
       if (j == size_type(-1)) {
 	dal::bit_vector bv;
 	for (size_type i = 0; i < list_constraints.size(); ++i)
-	  if (gmm::abs((*(list_constraints[i]))(pt)) < 1E-8) bv.add(i);
+	  if (gmm::abs((*(list_constraints[i]))(pt)) < 1E-8*radius_cv)
+	    bv.add(i);
 	j = points.add(pt);
 	constraints_.push_back(bv);
 	radius_.push_back(r);
@@ -173,7 +198,8 @@ struct Chrono {
       if (j == size_type(-1)) {
 	dal::bit_vector bv;
 	for (size_type i = 0; i < list_constraints.size(); ++i)
-	  if (gmm::abs((*(list_constraints[i]))(pt)) < 1E-8) bv.add(i);
+	  if (gmm::abs((*(list_constraints[i]))(pt)) < 1E-8*radius_cv)
+	    bv.add(i);
 	j = points.add(pt);
 	constraints_.push_back(bv);
 	scalar_type r = min_curvature_radius_estimate(list_constraints,pt,bv);
@@ -223,9 +249,11 @@ struct Chrono {
       return retained_points;
     }
 
-    point_stock(const std::vector<const mesher_signed_distance*> &ls)
-      : points(dal::lexicographical_less<base_node,
-	       dal::approx_less<scalar_type> >(1E-7)), list_constraints(ls) {}
+    point_stock(const std::vector<const mesher_signed_distance*> &ls,
+		scalar_type rcv)
+      : points(gmm::lexicographical_less<base_node,
+	       gmm::approx_less<scalar_type> >(1E-7)), list_constraints(ls),
+	radius_cv(rcv) {}
   };
 
 
@@ -250,8 +278,7 @@ struct Chrono {
       return new mesher_prism_ref(n);
     }
     
-    DAL_THROW(to_be_done_error,
-	      "This element is not taken into account. Contact us");
+    GMM_ASSERT1(false, "This element is not taken into account. Contact us");
   }
 
 
@@ -264,12 +291,12 @@ struct Chrono {
 				    gmm::dense_matrix<size_type> &simplexes,
 				    std::vector<dal::bit_vector> &
 				    /* fixed_points_constraints */) {
-    double t0=dal::uclock_sec();
+    double t0=gmm::uclock_sec();
     if (noisy) cout << "running delaunay with " << fixed_points.size()
 		    << " points.." << std::flush;
     delaunay(fixed_points, simplexes);
     if (noisy) cout << " -> " << gmm::mat_ncols(simplexes)
-		    << " simplexes [" << dal::uclock_sec()-t0 << "sec]\n";
+		    << " simplexes [" << gmm::uclock_sec()-t0 << "sec]\n";
   }
 
   static bool intersects(const mesh_level_set::zone &z1, 
@@ -324,19 +351,24 @@ struct Chrono {
      This information is now refined for each sub-convex.
   */
   void mesh_level_set::find_zones_of_element(size_type cv,
-					     std::string &prezone) {
+					     std::string &prezone,
+					     scalar_type radius) {
     convex_info &cvi = cut_cv[cv];
     cvi.zones.clear();
     for (dal::bv_visitor i(cvi.pmsh->convex_index()); !i.finished();++i) {
-      std::string subz = prezone;
-      //cout << "prezone for convex " << cv << " : " << subz << endl;
-      for (size_type j = 0; j < level_sets.size(); ++j) {
-	if (subz[j] == '*' || subz[j] == '0') {
-	  int s = sub_simplex_is_not_crossed_by(cv, level_sets[j], i);
-	  subz[j] = (s < 0) ? '-' : ((s > 0) ? '+' : '0');
+      // If the sub element is too small, the zone is not taken into account
+      if (cvi.pmsh->convex_area_estimate(i) > 1e-8) {
+	std::string subz = prezone;
+	//cout << "prezone for convex " << cv << " : " << subz << endl;
+	for (size_type j = 0; j < level_sets.size(); ++j) {
+	  if (subz[j] == '*' || subz[j] == '0') {
+	    int s = sub_simplex_is_not_crossed_by(cv, level_sets[j], i,radius);
+	    // cout << "sub_simplex_is_not_crossed_by = " << s << endl;
+	    subz[j] = (s < 0) ? '-' : ((s > 0) ? '+' : '0');
+	  }
 	}
+	merge_zoneset(cvi.zones, subz);
       }
-      merge_zoneset(cvi.zones, subz);
     }
     if (noisy) cout << "Number of zones for convex " << cv << " : "
 		    << cvi.zones.size() << endl;
@@ -345,7 +377,8 @@ struct Chrono {
 
   void mesh_level_set::cut_element(size_type cv,
 				   const dal::bit_vector &primary,
-				   const dal::bit_vector &secondary) {
+				   const dal::bit_vector &secondary,
+				   scalar_type radius_cv) {
     
     cut_cv[cv] = convex_info();
     cut_cv[cv].pmsh = pmesh(new mesh);
@@ -357,8 +390,7 @@ struct Chrono {
     size_type n = pgt->structure()->dim();
     size_type nbtotls = primary.card() + secondary.card();
     pintegration_method exactint = classical_exact_im(pgt);
-    if (nbtotls > 16)
-      DAL_THROW(failure_error,
+    GMM_ASSERT1(nbtotls <= 16,
 		"An element is cut by more than 16 level_set, aborting");
 
 		/* 
@@ -368,19 +400,21 @@ struct Chrono {
     dim_type K = 0; // Max. degree of the level sets.
     scalar_type r0 = 1E+10; // min curvature radius
     std::vector<const mesher_signed_distance*> list_constraints;
-    point_stock mesh_points(list_constraints);
+    point_stock mesh_points(list_constraints, radius_cv);
 
     ref_element->register_constraints(list_constraints);
     size_type nbeltconstraints = list_constraints.size();
     mesher_level_sets.reserve(nbtotls);
     for (size_type ll = 0; ll < level_sets.size(); ++ll) {
       if (primary[ll]) {
-	base_node X(n);
+	base_node X(n); gmm::fill_random(X);
 	K = std::max(K, (level_sets[ll])->degree());
 	mesher_level_sets.push_back(level_sets[ll]->mls_of_convex(cv, 0));
 	mesher_level_set &mls(mesher_level_sets.back());
 	list_constraints.push_back(&mesher_level_sets.back());
 	r0 = std::min(r0, curvature_radius_estimate(mls, X, true));
+	GMM_ASSERT1(gmm::abs(r0) >= 1e-13, "Something wrong in your level "
+		    "set ... curvature radius = " << r0);
 	if (secondary[ll]) {
 	  mesher_level_sets.push_back(level_sets[ll]->mls_of_convex(cv, 1));
 	  mesher_level_set &mls2(mesher_level_sets.back());
@@ -494,7 +528,7 @@ struct Chrono {
       msh.clear(); 
 
       for (size_type i = 0; i <  fixed_points.size(); ++i) {
-	size_type j = msh.add_point(fixed_points[i], false);
+	size_type j = msh.add_point(fixed_points[i]); // remettre le add_norepeat ?
 	assert(j == i);
       }
 
@@ -511,7 +545,7 @@ struct Chrono {
 	    for (size_type jj = 0; jj < list_constraints.size(); ++jj) {
 	      scalar_type dd =
 		(*(list_constraints[jj]))(msh.points_of_convex(j)[ii]);
-	      if (gmm::abs(dd) > 1E-7) {
+	      if (gmm::abs(dd) > radius_cv * 1E-7) {
 		if (dd * signs[jj] < 0.0) {
 		  if (noisy) cout << "Intersection trouvee ... \n";
 		  // calcul d'intersection
@@ -520,15 +554,28 @@ struct Chrono {
 		  if (dd > 0.) gmm::scale(VV, -1.);
 		  dd = (*(list_constraints[jj])).grad(X, G);
 		  size_type nbit = 0;
-		  while (gmm::abs(dd) > 1e-15) {
-		    if (++nbit > 1000) {
-		      if (noisy) cout << "Intersection not found";
-		      assert(false);
-		    }
-		    scalar_type nG = std::max(1E-8, gmm::vect_sp(G, VV));
+		  while (gmm::abs(dd) > 1e-15*radius_cv && (++nbit < 20)) { // Newton
+		    scalar_type nG = gmm::vect_sp(G, VV);
+		    if (gmm::abs(nG) < 1E-8) nG = 1E-8;
+		    if (nG < 0) nG = 1.0;
 		    gmm::add(gmm::scaled(VV, -dd / nG), X);
 		    dd = (*(list_constraints[jj])).grad(X, G);
 		  }
+		  if (gmm::abs(dd) > 1e-15*radius_cv) { // brute force dychotomie
+		    base_node X1 = msh.points_of_convex(j)[ii];
+		    base_node X2 = msh.points_of_convex(j)[prev_point[jj]], X3;
+		    scalar_type dd1 = (*(list_constraints[jj]))(X1);
+		    scalar_type dd2 = (*(list_constraints[jj]))(X2);
+		    if (dd1 > dd2) { std::swap(dd1, dd2); std::swap(X1, X2); }
+		    while (gmm::abs(dd1) > 1e-15*radius_cv) {
+		      X3 = (X1 + X2) / 2.0;
+		      scalar_type dd3 = (*(list_constraints[jj]))(X3);
+		      if (dd3 > 0) { dd2 = dd3; X2 = X3; }
+		      else { dd1 = dd3; X1 = X3; }
+		    }
+		    X = X1;
+		  }
+		  
 		  size_type kk = mesh_points.add(X);
 		  if (!(retained_points[kk])) {
 		    retained_points.add(kk);
@@ -694,8 +741,8 @@ struct Chrono {
 	
 	for (dal::bv_visitor i(msh.convex_index()); !i.finished(); ++i)
 	  global_mesh().add_convex(msh.trans_of_convex(i), 
-				 dal::index_ref_iterator(pts.begin(),
-				     msh.ind_points_of_convex(i).begin()));
+				   gmm::index_ref_iterator(pts.begin(),
+				   msh.ind_points_of_convex(i).begin()));
       }
 
     } while (!h0_is_ok);
@@ -725,7 +772,7 @@ struct Chrono {
 
 	for (dal::bv_visitor i(msh.convex_index()); !i.finished(); ++i) {
 	  ic2[i] = m.add_convex(msh.trans_of_convex(i), 
-				dal::index_ref_iterator(pts.begin(),
+				gmm::index_ref_iterator(pts.begin(),
 			        msh.ind_points_of_convex(i).begin()));
 	  
 	}
@@ -782,14 +829,15 @@ struct Chrono {
     std::string z;
     for (dal::bv_visitor cv(linked_mesh().convex_index()); 
 	 !cv.finished(); ++cv) {
+      scalar_type radius = linked_mesh().convex_radius_estimate(cv);
       dal::bit_vector prim, sec;
-      find_crossing_level_set(cv, prim, sec, z);
+      find_crossing_level_set(cv, prim, sec, z, radius);
       zones_of_convexes[cv] = &(*(allsubzones.insert(z).first));
       if (noisy) cout << "element " << cv << " cut level sets : "
 		      << prim << " zone : " << z << endl;
       if (prim.card()) {
-	cut_element(cv, prim, sec);
-	find_zones_of_element(cv, z);
+	cut_element(cv, prim, sec, radius);
+	find_zones_of_element(cv, z, radius);
       }
     }
     if (noisy) {
@@ -805,18 +853,25 @@ struct Chrono {
     is_adapted_ = true;
   }
   
+  // return +1 if the sub-element is on the one side of the level-set
+  //        -1 if the sub-element is on the other side of the level-set
+  //         0 if the sub-element is one the positive part of the secundary
+  //           level-set if any.
   int mesh_level_set::sub_simplex_is_not_crossed_by(size_type cv,
 						    plevel_set ls,
-						    size_type sub_cv) {
-    scalar_type EPS = 1e-8;
+						    size_type sub_cv,
+						    scalar_type radius) {
+    scalar_type EPS = 1e-7 * radius;
     bgeot::pgeometric_trans pgt = linked_mesh().trans_of_convex(cv);
     convex_info &cvi = cut_cv[cv];
     bgeot::pgeometric_trans pgt2 = cvi.pmsh->trans_of_convex(sub_cv);
 
+    // cout << "cv " << cv << " radius = " << radius << endl;
+
     mesher_level_set mls0 = ls->mls_of_convex(cv, 0), mls1(mls0);
     if (ls->has_secondary()) mls1 = ls->mls_of_convex(cv, 1);
     int p = 0;
-    bool cutted = false;
+    bool is_cut = false;
     scalar_type d2 = 0, d1 = 1, d0 = 0, d0min = 0;
     for (size_type i = 0; i < pgt2->nb_points(); ++i) {
       d0 = mls0(cvi.pmsh->points_of_convex(sub_cv)[i]);
@@ -828,25 +883,22 @@ struct Chrono {
       int p2 = ( (d0 < -EPS) ? -1 : ((d0 > EPS) ? +1 : 0));
       if (p == 0) p = p2;
       if (gmm::abs(d0) > gmm::abs(d2)) d2 = d0;
-      if (!p2 || p*p2 < 0) cutted = true;
+      if (!p2 || p*p2 < 0) is_cut = true;
     }
-    if (cutted && d1 > +EPS)
-      { cout << "ooops, je retourne 0: d1 = " << d1 << endl; return 0; }
-    if (d0min < EPS &&  d1 > -EPS) return 0;
+    // cout << "d0min = " << d0min << " d1 = " << d1 << " iscut = "
+    //      << is_cut << endl;
+    if (is_cut && ls->has_secondary() && d1 >= -radius*0.0001) return 0;
+    // if (d0min < EPS && ls->has_secondary() && d1 >= -EPS) return 0;
     return (d2 < 0.) ? -1 : 1;
   }
 
-
   int mesh_level_set::is_not_crossed_by(size_type cv, plevel_set ls,
-					unsigned lsnum) {
+					unsigned lsnum, scalar_type radius) {
     const mesh_fem &mf = ls->get_mesh_fem();
-    bgeot::pgeometric_trans pgt = linked_mesh().trans_of_convex(cv);
-
     const mesh_fem::ind_dof_ct &dofs = mf.ind_dof_of_element(cv);
     pfem pf = mf.fem_of_element(cv);
     int p = -2;
-    mesher_level_set mls1 = ls->mls_of_convex(cv, lsnum, false);
-    scalar_type EPS = 1e-8;
+    scalar_type EPS = 1e-8 * radius;
 
     /* easy cases: 
      - different sign on the dof nodes => intersection for sure
@@ -861,11 +913,13 @@ struct Chrono {
       if (!p2 || p*p2 < 0) return 0;
     }
 
+    mesher_level_set mls1 = ls->mls_of_convex(cv, lsnum, false);
     base_node X(pf->dim()), G(pf->dim());
     gmm::fill_random(X); X *= 1E-2;
     scalar_type d = mls1.grad(X, G);
-    if (gmm::vect_norm2(G)*2. < gmm::abs(d)) return p;
+    if (gmm::vect_norm2(G)*2.5 < gmm::abs(d)) return p;
 
+    bgeot::pgeometric_trans pgt = linked_mesh().trans_of_convex(cv);
     std::auto_ptr<mesher_signed_distance> ref_element(new_ref_element(pgt));
     
     gmm::fill_random(X); X *= 1E-2;
@@ -885,18 +939,19 @@ struct Chrono {
   void mesh_level_set::find_crossing_level_set(size_type cv,
 					       dal::bit_vector &prim,
 					       dal::bit_vector &sec,
-					       std::string &z) {
+					       std::string &z,
+					       scalar_type radius) {
     prim.clear(); sec.clear();
     z = std::string(level_sets.size(), '*');
     unsigned lsnum = 0;
     for (size_type k = 0; k < level_sets.size(); ++k, ++lsnum) {
       if (noisy) cout << "testing cv " << cv << " with level set "
 		      << k << endl;
-      int s = is_not_crossed_by(cv, level_sets[k], 0);
+      int s = is_not_crossed_by(cv, level_sets[k], 0, radius);
       if (!s) {
 	if (noisy) cout << "is cut \n";
 	if (level_sets[k]->has_secondary()) {
-	  s = is_not_crossed_by(cv, level_sets[k], 1);
+	  s = is_not_crossed_by(cv, level_sets[k], 1, radius);
 	  if (!s) { sec.add(lsnum); prim.add(lsnum); }
 	  else if (s < 0) prim.add(lsnum); else z[k] = '0';
 	}
