@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2004-2006 Julien Pommier
+// Copyright (C) 2004-2007 Julien Pommier
 //
 // This file is a part of GETFEM++
 //
@@ -20,7 +20,7 @@
 //
 //========================================================================
 
-#include <getfem_mesher.h>
+#include "getfem/getfem_mesher.h"
 
 namespace getfem {
 
@@ -69,14 +69,16 @@ namespace getfem {
 
   bool try_projection(const mesher_signed_distance& dist, base_node &X,
 		      bool on_surface) {
-    base_small_vector G;
+    base_small_vector G; base_node Y = X;
     scalar_type d = dist.grad(X, G), dmin = gmm::abs(d);
     size_type iter(0), count_falt(0);
     if (on_surface || d > 0.0)
-      while (gmm::abs(d) > 1e-15) {
+      while (iter == 0 || dmin > 1e-15 || gmm::vect_dist2(X, Y) > 1e-15 ) {
+	gmm::copy(X, Y);
 	if (++iter > 1000) {
-	  cout << "\n\nTry projection failed, 1000 iterations\n\n"; exit(1);
-	  return (gmm::abs(d) < 1E-10); // is there a possibility to detect
+	  GMM_WARNING4("Try projection failed, 1000 iterations\n\n");
+	  return false;
+	  // return (gmm::abs(d) < 1E-10); // is there a possibility to detect
 	} 	// the impossibility without making 1000 iterations ?
 	gmm::scale(G, -d / std::max(1E-8, gmm::vect_norm2_sqr(G)));
 	gmm::add(G, X);
@@ -90,12 +92,9 @@ namespace getfem {
 //	if (gmm::abs(alpha) < 1E-15) return (gmm::abs(d) < 1E-10);
 	d = dist.grad(X, G);
 	if (gmm::abs(d) >= dmin*0.95) ++count_falt;
-	else { count_falt=0; dmin = gmm::abs(d); }
-	if (count_falt > 20) return (gmm::abs(d) < 1E-10);
-//	cout.precision(16);
-//    	cout << "iter " << iter << " X = " << X << " dist = " << d << 
-//    	  " grad = " << G  << endl;
-
+	else { count_falt = 0; dmin = gmm::abs(d); }
+	// if (count_falt > 20) return (gmm::abs(d) < 1E-10);
+	if (count_falt > 20) return false;
       }
     return true;
   }
@@ -147,12 +146,12 @@ namespace getfem {
       for (i = 0; i < nbco; ++i) d[i] = -(ls[i]->grad(X, G[i]));
       ++iter;
       residual = gmm::vect_norm2(d);
-    } while (residual > 1e-14 && gmm::vect_dist2(oldX,X) > 1e-14
+    } while ((residual > 1e-14 || gmm::vect_dist2(oldX,X) > 1e-14)
 	     && iter < 1000);
 //     cout << "nb iter de pure_multi : " << iter
 //   	 << " norm(d) = " << gmm::vect_norm2(d) << " cts = " << cts << endl;
     for (i = 0; i < nbco; ++i) if (gmm::abs(d[i]) > SEPS) {
-      cout << "PURE MULTI HAS FAILED for " << cts << " nb iter = " << iter << endl;
+      //cout << "PURE MULTI HAS FAILED for " << cts << " nb iter = " << iter << endl;
       return false;
     }
     return true;
@@ -165,7 +164,7 @@ namespace getfem {
   // symmetric base_matrix
   static scalar_type max_vp(const base_matrix& M) {
     size_type m = mat_nrows(M);
-    if (!is_hermitian(M)) DAL_THROW(failure_error, "Matrix is not symmetric");
+    GMM_ASSERT1(is_hermitian(M), "Matrix is not symmetric");
     std::vector<scalar_type> eig(m);
     gmm::symmetric_qr_algorithm(M, eig);
     scalar_type emax(0);
@@ -389,7 +388,7 @@ namespace getfem {
       if (noisy > 0) cout << "Quality post-optimization\n";
       base_vector X(pts.size() * N);
       for (unsigned i=0; i < pts.size(); ++i)
-	dal::copy_n(pts[i].const_begin(), N, X.begin() + i*N);
+	gmm::copy_n(pts[i].const_begin(), N, X.begin() + i*N);
 
       base_matrix S(N,N), SW(N,N);
       for (unsigned i=0; i < nbt; ++i) {
@@ -428,7 +427,7 @@ namespace getfem {
 			  << " worst element: " << sqrt(worst_element) << endl;
 
       for (unsigned i=0; i < pts.size(); ++i)
-	dal::copy_n(X.begin() + i*N, N, pts[i].begin());      
+	gmm::copy_n(X.begin() + i*N, N, pts[i].begin());      
     }
 
     void delete_element(size_type ic) {
@@ -440,7 +439,7 @@ namespace getfem {
     }
 
     scalar_type quality_of_element(size_type i) {
-      return simplex_quality(dal::index_ref_iterator
+      return simplex_quality(gmm::index_ref_iterator
 			     (pts.begin(), gmm::mat_col(t,i).begin()));
     }
 
@@ -533,8 +532,8 @@ namespace getfem {
       size_type it(0);
       if (d > 0.0)
 	while (gmm::abs(d) > 1e-10) {
-	  if (++it > 10000)
-	    DAL_THROW(failure_error, "Object empty, or bad signed distance");
+	  ++it;
+	  GMM_ASSERT1(it <= 10000, "Object empty, or bad signed distance");
 // 	  cout << "iter " << it << " X = " << X << " dist = " << d << 
 // 	    " grad = " << G << endl;
 	  gmm::add(gmm::scaled(G, -d / gmm::vect_norm2_sqr(G)), X);
@@ -547,12 +546,12 @@ namespace getfem {
       scalar_type d = dist.grad(X, G);
       size_type it(0);
       while (gmm::abs(d) > 1e-10) {
-	if (++it > 10000)
-	    DAL_THROW(failure_error,
-		      "Object empty, or bad signed distance X=" << X << ", G=" << G << " d = " << d);
- 	if (it > 9980) 
-	  cout << "iter " << it << " X = " << X << " dist = " << d << 
- 	    " grad = " << G << endl;
+	++it;
+	GMM_ASSERT1(it <= 10000, "Object empty, or bad signed distance X="
+		    << X << ", G=" << G << " d = " << d);
+//  	if (it > 9980) 
+// 	  cout << "iter " << it << " X = " << X << " dist = " << d << 
+//  	    " grad = " << G << endl;
 	gmm::add(gmm::scaled(G, -d / gmm::vect_norm2_sqr(G)), X);
 	d = dist.grad(X, G);
       }
@@ -708,8 +707,7 @@ namespace getfem {
 
       for (size_type i=0; i < N; ++i) 
 	h0 = std::min(h0, bounding_box_max[i] - bounding_box_min[i]);
-      if (h0 < 1E-10)
-	DAL_THROW(failure_error, "h0 = " << h0 << " too small, aborting.");
+      GMM_ASSERT1(h0 >= 1E-10, "h0 = " << h0 << " too small, aborting.");
 
       for (size_type i=0; i < N; ++i) {
 	scalar_type h = h0;
@@ -1188,7 +1186,7 @@ namespace getfem {
 
 	// computation of L and L0.
 	size_type nbcv = edges_mesh.convex_index().card();
-	if (nbcv == 0) DAL_THROW(dal::failure_error, "no more edges!");
+	GMM_ASSERT1(nbcv != 0, "no more edges!");
 	L.resize(nbcv); L0.resize(nbcv);
 	scalar_type sL = 0, sL0 = 0;
 	for (dal::bv_visitor ie(edges_mesh.convex_index());
@@ -1373,8 +1371,8 @@ namespace getfem {
 # ifndef GETFEM_HAVE_QHULL_QHULL_H
   void delaunay(const std::vector<base_node> &,
 		gmm::dense_matrix<size_type>&) {
-    DAL_THROW(failure_error, "Qhull header files not installed. "
-	      "Install qhull library and reinstall Getfem++ library.");
+    GMM_ASSERT1(false, "Qhull header files not installed. "
+		"Install qhull library and reinstall Getfem++ library.");
   }
 # else
 
