@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2002-2006 Yves Renard, Julien Pommier.
+// Copyright (C) 2002-2007 Yves Renard, Julien Pommier.
 //
 // This file is a part of GETFEM++
 //
@@ -25,20 +25,21 @@
  *
  * This program is used to check that getfem++ is working. This is also 
  * a good example of use of Getfem++.
-*/
+ */
 
-#include <getfem_assembling.h> /* import assembly methods (and norms comp.) */
-#include <getfem_export.h>   /* export functions (save solution in a file)  */
-#include <getfem_derivatives.h>
-#include <getfem_regular_meshes.h>
-#include <getfem_model_solvers.h>
-#include <getfem_mesh_im_level_set.h>
-#include <getfem_mesh_fem_level_set.h>
-#include <getfem_mesh_fem_product.h>
-#include <getfem_mesh_fem_global_function.h>
-#include <getfem_spider_fem.h>
-#include <getfem_mesh_fem_sum.h>
-#include <gmm.h>
+#include "getfem/getfem_assembling.h" /* import assembly methods (and norms comp.) */
+#include "getfem/getfem_export.h"   /* export functions (save solution in a file)  */
+#include "getfem/getfem_derivatives.h"
+#include "getfem/getfem_regular_meshes.h"
+#include "getfem/getfem_model_solvers.h"
+#include "getfem/getfem_mesh_im_level_set.h"
+#include "getfem/getfem_mesh_fem_level_set.h"
+#include "getfem/getfem_mesh_fem_product.h"
+#include "getfem/getfem_mesh_fem_global_function.h"
+#include "getfem/getfem_spider_fem.h"
+#include "getfem/getfem_mesh_fem_sum.h"
+#include "gmm/gmm.h"
+#include "gmm/gmm_inoutput.h"
 
 /* some Getfem++ types that we will be using */
 using bgeot::base_small_vector; /* special class for small (dim<16) vectors */
@@ -241,12 +242,15 @@ struct exact_solution {
   void init(int mode, scalar_type lambda, scalar_type mu,
 	    getfem::level_set &ls) {
     std::vector<getfem::pglobal_function> cfun(4);
-    for (unsigned j=0; j < 4; ++j)
-      cfun[j] = getfem::isotropic_crack_singular_2D(j, ls);
+    for (unsigned j=0; j < 4; ++j) {
+      getfem::crack_singular_xy_function *s = 
+	new getfem::crack_singular_xy_function(j);
+      cfun[j] = getfem::global_function_on_level_set(ls, *s);
+    }
+
     mf.set_functions(cfun);
     
     mf.set_qdim(1);
-   
     
     U.resize(8); assert(mf.nb_dof() == 4);
     getfem::base_vector::iterator it = U.begin();
@@ -299,22 +303,20 @@ base_small_vector sol_f(const base_node &x) {
 
 struct crack_problem {
 
-  enum { DIRICHLET_BOUNDARY_NUM = 0, NEUMANN_BOUNDARY_NUM = 1, NEUMANN_BOUNDARY_NUM1=2, NEUMANN_HOMOGENE_BOUNDARY_NUM=3};
+  enum { DIRICHLET_BOUNDARY_NUM = 0, NEUMANN_BOUNDARY_NUM = 1, MORTAR_BOUNDARY_IN=42, MORTAR_BOUNDARY_OUT=43 };
   getfem::mesh mesh;  /* the mesh */
   getfem::mesh_level_set mls;       /* the integration methods.              */
   getfem::mesh_im_level_set mim;    /* the integration methods.              */
-  getfem::mesh_fem mf_pre_u;
+  getfem::mesh_fem mf_pre_u, mf_pre_mortar;
   getfem::mesh_fem mf_mult;
-  getfem::mesh_fem_level_set mfls_u; 
+  getfem::mesh_fem_level_set mfls_u, mfls_mortar; 
   getfem::mesh_fem_global_function mf_sing_u;
   getfem::mesh_fem mf_partition_of_unity;
   getfem::mesh_fem_product mf_product;
   getfem::mesh_fem_sum mf_u_sum;
-  getfem::spider_fem *spider;
   getfem::mesh_fem mf_us;
 
   getfem::mesh_fem& mf_u() { return mf_u_sum; }
-  // getfem::mesh_fem& mf_u() { return mf_us; }
   
   scalar_type lambda, mu;    /* Lame coefficients.                */
   getfem::mesh_fem mf_rhs;   /* mesh_fem for the right hand side (f(x),..)   */
@@ -323,30 +325,55 @@ struct crack_problem {
   exact_solution exact_sol;
 #endif
   
-  
-  int bimaterial;           /* For bimaterial interface fracture */
-  double lambda_up, lambda_down;  /*Lame coeff for bimaterial case*/
   getfem::level_set ls;      /* The two level sets defining the crack.       */
   getfem::level_set ls2, ls3; /* The two level-sets defining the add. cracks.*/
+
+  dal::bit_vector pm_convexes; /* convexes inside the enrichment 
+				  area when point-wise matching is used.*/
+
   base_small_vector translation;
-  scalar_type theta0;
-  scalar_type spider_radius;
-  unsigned spider_Nr;
-  unsigned spider_Ntheta;
-  int spider_K;
+
+  struct spider_param {
+    getfem::spider_fem *fem;
+    scalar_type theta0;
+    scalar_type radius;
+    unsigned Nr;
+    unsigned Ntheta;
+    int K;
+  };
+  spider_param spider;
+
   scalar_type residual;      /* max residual for the iterative solvers      */
   bool mixed_pressure, add_crack;
   unsigned dir_with_mult;
-  scalar_type cutoff_radius, cutoff_radius1, cutoff_radius0, enr_area_radius;
-  int enrichment_option;
-  size_type cutoff_func;
+  
+  scalar_type enr_area_radius;
+  struct cutoff_param {
+    scalar_type radius, radius1, radius0;
+    size_type fun_num;
+  };
+  cutoff_param cutoff;
+
+  typedef enum { NO_ENRICHMENT=0, 
+		 FIXED_ZONE=1, 
+		 GLOBAL_WITH_MORTAR=2,
+		 GLOBAL_WITH_CUTOFF=3,
+		 SPIDER_FEM_ALONE=4,
+		 SPIDER_FEM_ENRICHMENT=5 } enrichment_option_enum;
+  enrichment_option_enum enrichment_option;
+
   std::string datafilename;
-  ftool::md_param PARAM;
+  
+  std::string GLOBAL_FUNCTION_MF, GLOBAL_FUNCTION_U;
+
+  bgeot::md_param PARAM;
 
   bool solve(plain_vector &U);
   void init(void);
-  crack_problem(void) : mls(mesh), mim(mls), mf_pre_u(mesh), mf_mult(mesh),
-			mfls_u(mls, mf_pre_u), mf_sing_u(mesh),
+  crack_problem(void) : mls(mesh), mim(mls), 
+			mf_pre_u(mesh), mf_pre_mortar(mesh), mf_mult(mesh),
+			mfls_u(mls, mf_pre_u), mfls_mortar(mls, mf_pre_mortar),
+			mf_sing_u(mesh),
 			mf_partition_of_unity(mesh),
 			mf_product(mf_partition_of_unity, mf_sing_u),
 
@@ -359,6 +386,44 @@ struct crack_problem {
 
 };
 
+
+std::string name_of_dof(getfem::pdof_description dof) {
+  char s[200];
+  sprintf(s, "UnknownDof[%p]", (void*)dof);
+  for (unsigned d = 0; d < 4; ++d) {
+    if (dof == getfem::lagrange_dof(d)) {
+      sprintf(s, "Lagrange[%d]", d); goto found;
+    }
+    if (dof == getfem::normal_derivative_dof(d)) {
+      sprintf(s, "D_n[%d]", d); goto found;
+    }
+    if (dof == getfem::global_dof(d)) {
+      sprintf(s, "GlobalDof[%d]", d);
+    }
+    if (dof == getfem::mean_value_dof(d)) {
+      sprintf(s, "MeanValue[%d]", d);
+    }
+    if (getfem::dof_xfem_index(dof) != 0) {
+      sprintf(s, "Xfem[idx:%d]", int(dof_xfem_index(dof)));
+    }
+    
+    for (unsigned r = 0; r < d; ++r) {
+      if (dof == getfem::derivative_dof(d, r)) {
+	sprintf(s, "D_%c[%d]", "xyzuvw"[r], d); goto found;
+      }
+      for (unsigned t = 0; t < d; ++t) {
+	if (dof == getfem::second_derivative_dof(d, r, t)) {
+	  sprintf(s, "D2%c%c[%d]", "xyzuvw"[r], "xyzuvw"[t], d); 
+	  goto found;
+	}
+      }
+    }
+  }
+ found:
+  return s;
+}
+
+
 /* Read parameters from the .param file, build the mesh, set finite element
  * and integration methods and selects the boundaries.
  */
@@ -372,21 +437,22 @@ void crack_problem::init(void) {
   std::string SINGULAR_INTEGRATION = PARAM.string_value("SINGULAR_INTEGRATION");
 
   add_crack = (PARAM.int_value("ADDITIONAL_CRACK", "An additional crack ?") != 0);
-  enrichment_option = PARAM.int_value("ENRICHMENT_OPTION",
-				      "Enrichment option");
+  enrichment_option = 
+    enrichment_option_enum(PARAM.int_value("ENRICHMENT_OPTION",
+					   "Enrichment option"));
   cout << "MESH_TYPE=" << MESH_TYPE << "\n";
   cout << "FEM_TYPE="  << FEM_TYPE << "\n";
   cout << "INTEGRATION=" << INTEGRATION << "\n";
 
-  spider_radius = PARAM.real_value("SPIDER_RADIUS","spider_radius");
-  spider_Nr = PARAM.int_value("SPIDER_NR","Spider_Nr ");
-  spider_Ntheta = PARAM.int_value("SPIDER_NTHETA","Ntheta ");
-  spider_K = PARAM.int_value("SPIDER_K","K ");
+  spider.radius = PARAM.real_value("SPIDER_RADIUS","spider_radius");
+  spider.Nr = PARAM.int_value("SPIDER_NR","Spider_Nr ");
+  spider.Ntheta = PARAM.int_value("SPIDER_NTHETA","Ntheta ");
+  spider.K = PARAM.int_value("SPIDER_K","K ");
+  spider.theta0 =0;
 
   translation.resize(2); 
   translation[0] =0.5;
   translation[1] =0.;
-  theta0 =0;
   
   /* First step : build the mesh */
   bgeot::pgeometric_trans pgt = 
@@ -401,34 +467,18 @@ void crack_problem::init(void) {
   mesh.translation(tt); 
   
   datafilename = PARAM.string_value("ROOTFILENAME","Base name of data files.");
-  residual = PARAM.real_value("RESIDUAL"); if (residual == 0.) residual = 1e-10;
+  residual = PARAM.real_value("RESIDUAL");
+  if (residual == 0.) residual = 1e-10;
   enr_area_radius = PARAM.real_value("RADIUS_ENR_AREA",
 				     "radius of the enrichment area");
   
-  bimaterial = PARAM.int_value("BIMATERIAL", "bimaterial interface crack");
+  mu = PARAM.real_value("MU", "Lame coefficient mu");
+  lambda = PARAM.real_value("LAMBDA", "Lame coefficient lambda");
 
-
-  
-
-
-  if (bimaterial == 1){
-    mu = PARAM.real_value("MU", "Lame coefficient mu"); 
-    lambda_up = PARAM.int_value("LAMBDA_UP", "Lame Coef");
-    lambda_down = PARAM.int_value("LAMBDA_DOWN", "Lame Coef");
-    lambda = PARAM.real_value("LAMBDA", "Lame coefficient lambda");
-  }
-  else{
-
-    mu = PARAM.real_value("MU", "Lame coefficient mu");
-    lambda = PARAM.real_value("LAMBDA", "Lame coefficient lambda");
-  }
-  
-
-  cutoff_func = PARAM.int_value("CUTOFF_FUNC", "cutoff function");
-
-  cutoff_radius = PARAM.real_value("CUTOFF", "Cutoff");
-  cutoff_radius1 = PARAM.real_value("CUTOFF1", "Cutoff1");
-  cutoff_radius0 = PARAM.real_value("CUTOFF0", "Cutoff0");
+  cutoff.fun_num = PARAM.int_value("CUTOFF_FUNC", "cutoff function");
+  cutoff.radius = PARAM.real_value("CUTOFF", "Cutoff");
+  cutoff.radius1 = PARAM.real_value("CUTOFF1", "Cutoff1");
+  cutoff.radius0 = PARAM.real_value("CUTOFF0", "Cutoff0");
   mf_u().set_qdim(N);
 
   /* set the finite element on the mf_u */
@@ -447,22 +497,12 @@ void crack_problem::init(void) {
 
   mim.set_simplex_im(simp_ppi, sing_ppi);
   mf_pre_u.set_finite_element(mesh.convex_index(), pf_u);
+  mf_pre_mortar.set_finite_element(mesh.convex_index(), 
+				   getfem::fem_descriptor(PARAM.string_value("MORTAR_FEM_TYPE")));
   mf_mult.set_finite_element(mesh.convex_index(), pf_u);
   mf_mult.set_qdim(N);
   mf_partition_of_unity.set_classical_finite_element(1);
   
-//   if (enrichment_option == 3 || enrichment_option == 4) {
-//     spider = new getfem::spider_fem(spider_radius, mim, spider_Nr,
-// 				    spider_Ntheta, spider_K, translation,
-// 				    theta0);
-//     mf_us.set_finite_element(mesh.convex_index(),spider->get_pfem());
-//     for (dal::bv_visitor_c i(mf_us.convex_index()); !i.finished(); ++i) {
-//       if (mf_us.fem_of_element(i)->nb_dof(i) == 0) {
-// 	mf_us.set_finite_element(i,0);
-//       }
-//     }
-//   }
-
   mixed_pressure =
     (PARAM.int_value("MIXED_PRESSURE","Mixed version or not.") != 0);
   dir_with_mult = PARAM.int_value("DIRICHLET_VERSINO");
@@ -477,9 +517,9 @@ void crack_problem::init(void) {
   std::string data_fem_name = PARAM.string_value("DATA_FEM_TYPE");
   if (data_fem_name.size() == 0) {
     if (!pf_u->is_lagrange()) {
-      DAL_THROW(dal::failure_error, "You are using a non-lagrange FEM. "
-		<< "In that case you need to set "
-		<< "DATA_FEM_TYPE in the .param file");
+      GMM_ASSERT1(false, "You are using a non-lagrange FEM. "
+		  << "In that case you need to set "
+		  << "DATA_FEM_TYPE in the .param file");
     }
     mf_rhs.set_finite_element(mesh.convex_index(), pf_u);
   } else {
@@ -496,34 +536,9 @@ void crack_problem::init(void) {
     
     base_node un = mesh.normal_of_face_of_convex(i.cv(), i.f());
     un /= gmm::vect_norm2(un);
-    if(bimaterial == 1) {
-      
-      if (un[0]  > 1.0E-7 ) { // new Neumann face
-	mesh.region(DIRICHLET_BOUNDARY_NUM).add(i.cv(), i.f());
-      } else {
-	if (un[1]  > 1.0E-7 ) {
-	  cout << "normal = " << un << endl;
-	  mesh.region(NEUMANN_BOUNDARY_NUM1).add(i.cv(), i.f());
-	}
-	else {
-	  if (un[1]  < -1.0E-7 ) {
-	    cout << "normal = " << un << endl;
-	    mesh.region(NEUMANN_BOUNDARY_NUM).add(i.cv(), i.f());
-	  }
-	  else {
-	    if (un[0]  < -1.0E-7 ) {
-	      cout << "normal = " << un << endl;
-	      mesh.region(NEUMANN_HOMOGENE_BOUNDARY_NUM).add(i.cv(), i.f());
-	    }
-	  }
-	}
-      }
-    }
-    else {
-      
 #ifdef VALIDATE_XFEM
-      mesh.region(DIRICHLET_BOUNDARY_NUM).add(i.cv(), i.f());
-      #else
+    mesh.region(DIRICHLET_BOUNDARY_NUM).add(i.cv(), i.f());
+#else
     base_node un = mesh.normal_of_face_of_convex(i.cv(), i.f());
     un /= gmm::vect_norm2(un);
     if (un[0] - 1.0 < -1.0E-7) { // new Neumann face
@@ -531,9 +546,8 @@ void crack_problem::init(void) {
     } else {
       cout << "normal = " << un << endl;
       mesh.region(DIRICHLET_BOUNDARY_NUM).add(i.cv(), i.f());
-      }
-#endif
     }
+#endif
   }
   
   
@@ -595,41 +609,49 @@ bool crack_problem::solve(plain_vector &U) {
   mls.adapt();
   mim.adapt();
   mfls_u.adapt();
-  std::vector<getfem::pglobal_function> vfunc(4);
-  for (size_type i = 0; i < 4; ++i)
-    vfunc[i] = isotropic_crack_singular_2D(i, ls,
-					   (enrichment_option == 2) ? 0.0 : cutoff_radius,
-					   (enrichment_option == 2) ? 0.0 : cutoff_radius1,
-					   (enrichment_option == 2) ? 0.0 : cutoff_radius0,
-					   cutoff_func);
+  mfls_mortar.adapt(); mfls_mortar.set_qdim(2);
 
+  cout << "Setting up the singular functions for the enrichment\n";
+  std::vector<getfem::pglobal_function> vfunc(4);
+  for (size_type i = 0; i < vfunc.size(); ++i) {
+    /* use the singularity */
+    getfem::abstract_xy_function *s = 
+      new getfem::crack_singular_xy_function(i);
+    if (enrichment_option != FIXED_ZONE && 
+	enrichment_option != GLOBAL_WITH_MORTAR) {
+      /* use the product of the singularity function
+	 with a cutoff */
+      getfem::abstract_xy_function *c = 
+	new getfem::cutoff_xy_function(cutoff.fun_num,
+				       cutoff.radius, 
+				       cutoff.radius1,cutoff.radius0);
+      s = new getfem::product_of_xy_functions(*s, *c);
+    }
+    vfunc[i] = getfem::global_function_on_level_set(ls, *s);
+  }
   
   mf_sing_u.set_functions(vfunc);
 
 
-  if (enrichment_option == 3 || enrichment_option == 4) {
-    spider = new getfem::spider_fem(spider_radius, mim, spider_Nr,
-				    spider_Ntheta, spider_K, translation,
-				    theta0);
-    mf_us.set_finite_element(mesh.convex_index(),spider->get_pfem());
+  if (enrichment_option == SPIDER_FEM_ALONE || 
+      enrichment_option == SPIDER_FEM_ENRICHMENT) {
+    spider.fem = new getfem::spider_fem(spider.radius, mim, spider.Nr,
+					spider.Ntheta, spider.K, translation,
+					spider.theta0);
+    mf_us.set_finite_element(mesh.convex_index(),spider.fem->get_pfem());
     for (dal::bv_visitor_c i(mf_us.convex_index()); !i.finished(); ++i) {
       if (mf_us.fem_of_element(i)->nb_dof(i) == 0) {
 	mf_us.set_finite_element(i,0);
       }
     }
-    spider->check();
+    spider.fem->check();
   }
 
+
   switch (enrichment_option) {
-  case 1 :{
-    if(cutoff_func == 0)
-      cout<<"Using exponential Cutoff..."<<endl;
-    else
-   cout<<"Using Polynomial Cutoff..."<<endl;
-    mf_u_sum.set_mesh_fems(mf_sing_u, mfls_u); break;
-  }
-  case 2 :
-    {
+
+
+    case FIXED_ZONE: {
       dal::bit_vector enriched_dofs;
       plain_vector X(mf_partition_of_unity.nb_dof());
       plain_vector Y(mf_partition_of_unity.nb_dof());
@@ -642,17 +664,76 @@ bool crack_problem::solve(plain_vector &U) {
 	  enriched_dofs.add(j);
       }
       if (enriched_dofs.card() < 3)
-	DAL_WARNING0("There is " << enriched_dofs.card() <<
+	GMM_WARNING0("There is " << enriched_dofs.card() <<
 		     " enriched dofs for the crack tip");
       mf_product.set_enrichment(enriched_dofs);
       mf_u_sum.set_mesh_fems(mf_product, mfls_u);
-    }
-    break;
-    case 3 : mf_u_sum.set_mesh_fems(mf_us); break;
-  case 4 : 
-    mf_u_sum.set_mesh_fems(mf_us, mfls_u); 
-    break;
-  default : mf_u_sum.set_mesh_fems(mfls_u); break;
+    } break;
+
+    case GLOBAL_WITH_MORTAR: {
+      // Selecting the element in the enriched domain
+
+      dal::bit_vector cvlist_in_area;
+      dal::bit_vector cvlist_out_area;
+      for (dal::bv_visitor cv(mesh.convex_index()); 
+	   !cv.finished(); ++cv) {
+	bool in_area = true;
+	/* For each element, we test all of its nodes. 
+	   If all the nodes are inside the enrichment area,
+	   then the element is completly inside the area too */ 
+	for (unsigned j=0; j < mesh.nb_points_of_convex(cv); ++j) {
+	  if (gmm::sqr(mesh.points_of_convex(cv)[j][0] - translation[0]) + 
+	      gmm::sqr(mesh.points_of_convex(cv)[j][1] - translation[1]) > 
+	      gmm::sqr(enr_area_radius)) {
+	    in_area = false; break;
+	  }
+	}
+
+	/* "remove" the global function on convexes outside the enrichment
+	   area */
+	if (!in_area) {
+	  cvlist_out_area.add(cv);
+	  mf_sing_u.set_finite_element(cv, 0);
+	  mf_u().set_dof_partition(cv, 1);
+	} else cvlist_in_area.add(cv);
+      }
+
+      /* extract the boundary of the enrichment area, from the
+	 "inside" point-of-view, and from the "outside"
+	 point-of-view */
+      getfem::mesh_region r_border, r_enr_out;
+      getfem::outer_faces_of_mesh(mesh, r_border);
+
+      getfem::outer_faces_of_mesh(mesh, cvlist_in_area, 
+				  mesh.region(MORTAR_BOUNDARY_IN));
+      getfem::outer_faces_of_mesh(mesh, cvlist_out_area, 
+				  mesh.region(MORTAR_BOUNDARY_OUT));
+      for (getfem::mr_visitor v(r_border); !v.finished(); ++v) {
+	mesh.region(MORTAR_BOUNDARY_OUT).sup(v.cv(), v.f());
+      }
+      mf_u_sum.set_mesh_fems(mf_sing_u, mfls_u);
+    } break;
+
+    case GLOBAL_WITH_CUTOFF :{
+      if(cutoff.fun_num == getfem::cutoff_xy_function::EXPONENTIAL_CUTOFF)
+	cout<<"Using exponential Cutoff..."<<endl;
+      else
+	cout<<"Using Polynomial Cutoff..."<<endl;
+      mf_u_sum.set_mesh_fems(mf_sing_u, mfls_u);
+    } break;
+
+    case SPIDER_FEM_ALONE: {
+      mf_u_sum.set_mesh_fems(mf_us); 
+    } break;
+  
+    case SPIDER_FEM_ENRICHMENT: {
+      mf_u_sum.set_mesh_fems(mf_us, mfls_u); 
+    } break;
+
+    case NO_ENRICHMENT: {
+      mf_u_sum.set_mesh_fems(mfls_u); 
+    } break;
+  
   }
   
 
@@ -660,37 +741,44 @@ bool crack_problem::solve(plain_vector &U) {
 
   if (mixed_pressure) cout << "Number of dof for P: " << mf_p.nb_dof() << endl;
   cout << "Number of dof for u: " << mf_u().nb_dof() << endl;
+
+  unsigned Q = mf_u().get_qdim();
+  if (0) {
+    for (unsigned d=0; d < mf_u().nb_dof(); d += Q) {
+      printf("dof %4d @ %+6.2f:%+6.2f: ", d, 
+	     mf_u().point_of_dof(d)[0], mf_u().point_of_dof(d)[1]);
+
+      const getfem::mesh::ind_cv_ct cvs = mf_u().convex_to_dof(d);
+      for (unsigned i=0; i < cvs.size(); ++i) {
+	unsigned cv = cvs[i];
+	//if (pm_cvlist.is_in(cv)) flag1 = true; else flag2 = true;
+
+	getfem::pfem pf = mf_u().fem_of_element(cv);
+	unsigned ld = unsigned(-1);
+	for (unsigned dd = 0; dd < mf_u().nb_dof_of_element(cv); dd += Q) {
+	  if (mf_u().ind_dof_of_element(cv)[dd] == d) {
+	    ld = dd/Q; break;
+	  }
+	}
+	if (ld == unsigned(-1)) {
+	  cout << "DOF " << d << "NOT FOUND in " << cv << " BUG BUG\n";
+	} else {
+	  printf(" %3d:%.16s", cv, name_of_dof(pf->dof_types().at(ld)).c_str());
+	}
+      }
+      printf("\n");
+    }
+  }
   
   // Linearized elasticity brick.
   
   
   getfem::mdbrick_isotropic_linearized_elasticity<>
     ELAS(mim, mf_u(), mixed_pressure ? 0.0 : lambda, mu);
-
-  
-  if(bimaterial == 1){
-    cout<<"______________________________________________________________________________"<<endl;
-    cout<<"CASE OF BIMATERIAL CRACK  with lambda_up = "<<lambda_up<<" and lambda_down = "<<lambda_down<<endl;
-    cout<<"______________________________________________________________________________"<<endl;
-    std::vector<float> bi_lambda(ELAS.lambda().mf().nb_dof());
-    
-    cout<<"ELAS.lambda().mf().nb_dof()==="<<ELAS.lambda().mf().nb_dof()<<endl;
-    
-    for (size_type ite = 0; ite < ELAS.lambda().mf().nb_dof();ite++) {
-      if (ELAS.lambda().mf().point_of_dof(ite)[1] > 0)
-	bi_lambda[ite] = lambda_up;
-	else
-	  bi_lambda[ite] = lambda_down;
-    }
-    
-    //cout<<"bi_lambda.size() = "<<bi_lambda.size()<<endl;
-    // cout<<"ELAS.lambda().mf().nb_dof()==="<<ELAS.lambda().mf().nb_dof()<<endl;
-    
-    ELAS.lambda().set(bi_lambda);
-  }
   
+  //gmm::HarwellBoeing_IO::write("K.hb", ELAS.get_K());
 
-    getfem::mdbrick_abstract<> *pINCOMP;
+  getfem::mdbrick_abstract<> *pINCOMP;
   if (mixed_pressure) {
     getfem::mdbrick_linear_incomp<> *incomp
       = new getfem::mdbrick_linear_incomp<>(ELAS, mf_p);
@@ -711,60 +799,107 @@ bool crack_problem::solve(plain_vector &U) {
   gmm::clear(F);
   
   // Neumann condition brick.
-  
-  getfem::mdbrick_abstract<> *pNEUMANN;
-    
-
-  if(bimaterial ==  1){
-  for(size_type i = 1; i<F.size(); i=i+2) 
-    F[i]=-0.2;
-  }
-  
-  getfem::mdbrick_source_term<>  NEUMANN(VOL_F, mf_rhs, F,NEUMANN_BOUNDARY_NUM);   
-  
-   gmm::clear(F);
-   getfem::mdbrick_source_term<> NEUMANN_HOM(NEUMANN, mf_rhs, F,NEUMANN_HOMOGENE_BOUNDARY_NUM);
-  
-  
-  gmm::clear(F);
-  for(size_type i = 1; i<F.size(); i=i+2) 
-    F[i]=0.2;
- 
-  getfem::mdbrick_source_term<> NEUMANN1(NEUMANN_HOM, mf_rhs, F,NEUMANN_BOUNDARY_NUM1);
-  
-  if (bimaterial ==1)
-    pNEUMANN = & NEUMANN1;
-  else
-    pNEUMANN = & NEUMANN;
-   
-    
+        
+  getfem::mdbrick_source_term<>  NEUMANN(VOL_F, mf_rhs, F,NEUMANN_BOUNDARY_NUM);        
    
  //toto_solution toto(mf_rhs.linked_mesh()); toto.init();
  //assert(toto.mf.nb_dof() == 1);
    
  // Dirichlet condition brick.
-   getfem::mdbrick_Dirichlet<> final_model(*pNEUMANN, DIRICHLET_BOUNDARY_NUM, mf_mult);
-   
-   if (bimaterial == 1)
-     final_model.rhs().set(exact_sol.mf,0);
-   else {
+  getfem::mdbrick_Dirichlet<> DIRICHLET(NEUMANN, DIRICHLET_BOUNDARY_NUM, mf_mult);
+  
 #ifdef VALIDATE_XFEM
-     final_model.rhs().set(exact_sol.mf,exact_sol.U);
-#endif
-   }
+  DIRICHLET.rhs().set(exact_sol.mf,exact_sol.U);
+#endif   
+  DIRICHLET.set_constraints_type(getfem::constraints_type(dir_with_mult));
+
+  getfem::mdbrick_abstract<> *final_model = &DIRICHLET;
+
+  if (enrichment_option == GLOBAL_WITH_MORTAR) {
+    /* add a constraint brick for the mortar junction between
+       the enriched area and the rest of the mesh */
+    /* we use mfls_u as the space of lagrange multipliers */
+    getfem::mesh_fem &mf_mortar = mfls_mortar; 
+    /* adjust its qdim.. this is just evil and dangerous
+       since mf_u() is built upon mfls_u.. it would be better
+       to use a copy. */
+    mf_mortar.set_qdim(2); // EVIL 
+    getfem::mdbrick_constraint<> &mortar = 
+      *(new getfem::mdbrick_constraint<>(DIRICHLET,0));
+
+    cout << "Handling mortar junction\n";
+
+    /* list of dof of mf_mortar for the mortar condition */
+    std::vector<size_type> ind_mortar;
+    /* unfortunately , dof_on_region sometimes returns too much dof
+       when mf_mortar is an enriched one so we have to filter them */
+    sparse_matrix M(mf_mortar.nb_dof(), mf_mortar.nb_dof());
+    getfem::asm_mass_matrix(M, mim, mf_mortar, MORTAR_BOUNDARY_OUT);
+    for (dal::bv_visitor_c d(mf_mortar.dof_on_region(MORTAR_BOUNDARY_OUT)); 
+	 !d.finished(); ++d) {
+      if (M(d,d) > 1e-8) ind_mortar.push_back(d);
+      else cout << "  removing non mortar dof" << d << "\n";
+    }
+
+    cout << ind_mortar.size() << " dof for the lagrange multiplier)\n";
+
+    sparse_matrix H0(mf_mortar.nb_dof(), mf_u().nb_dof()), 
+      H(ind_mortar.size(), mf_u().nb_dof());
+
+
+    gmm::sub_index sub_i(ind_mortar);
+    gmm::sub_interval sub_j(0, mf_u().nb_dof());
+    
+    /* build the mortar constraint matrix -- note that the integration
+       method is conformal to the crack
+     */
+    getfem::asm_mass_matrix(H0, mim, mf_mortar, mf_u(), 
+			    MORTAR_BOUNDARY_OUT);
+    gmm::copy(gmm::sub_matrix(H0, sub_i, sub_j), H);
+
+    gmm::clear(H0);
+    getfem::asm_mass_matrix(H0, mim, mf_mortar, mf_u(), 
+			    MORTAR_BOUNDARY_IN);
+    gmm::add(gmm::scaled(gmm::sub_matrix(H0, sub_i, sub_j), -1), H);
+
+
+    /* because of the discontinuous partition of mf_u(), some levelset
+       enriched functions do not contribute any more to the
+       mass-matrix (the ones which are null on one side of the
+       levelset, when split in two by the mortar partition, may create
+       a "null" dof whose base function is all zero..
+    */
+    sparse_matrix M2(mf_u().nb_dof(), mf_u().nb_dof());
+    getfem::asm_mass_matrix(M2, mim, mf_u(), mf_u());
+
+    for (size_type d = 0; d < mf_u().nb_dof(); ++d) {
+      if (M2(d,d) < 1e-10) {
+	cout << "  removing null mf_u() dof " << d << "\n";
+	unsigned n = gmm::mat_nrows(H);
+	gmm::resize(H, n+1, gmm::mat_ncols(H));
+	H(n, d) = 1;
+      }
+    }
+    
+    
+    
+    getfem::base_vector R(gmm::mat_nrows(H));
+    mortar.set_constraints(H,R);
+
+    final_model = &mortar;
+  }
+
    
-   final_model.set_constraints_type(getfem::constraints_type(dir_with_mult));
-   
-   // Generic solve.
- cout << "Total number of variables : " << final_model.nb_dof() << endl;
- getfem::standard_model_state MS(final_model);
- gmm::iteration iter(residual, 1, 40000);
- getfem::standard_solve(MS, final_model, iter);
- 
- // Solution extraction
- gmm::copy(ELAS.get_solution(MS), U);
- 
- return (iter.converged());
+  // Generic solve.
+  cout << "Total number of variables : " << final_model->nb_dof() << endl;
+  getfem::standard_model_state MS(*final_model);
+  gmm::iteration iter(residual, 1, 40000);
+  getfem::standard_solve(MS, *final_model, iter);
+  
+  // Solution extraction
+  gmm::copy(ELAS.get_solution(MS), U);
+  
+  return (iter.converged());
 }
 
 /**************************************************************************/
@@ -773,7 +908,7 @@ bool crack_problem::solve(plain_vector &U) {
 
 int main(int argc, char *argv[]) {
 
-  DAL_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
+  GMM_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
   FE_ENABLE_EXCEPT;        // Enable floating point exception for Nan.
 
   //getfem::getfem_mesh_level_set_noisy();
@@ -784,10 +919,11 @@ int main(int argc, char *argv[]) {
     p.PARAM.read_command_line(argc, argv);
     p.init();
     p.mesh.write_to_file(p.datafilename + ".mesh");
-    plain_vector U(p.mf_u().nb_dof());
-    if (!p.solve(U)) DAL_THROW(dal::failure_error,"Solve has failed");
 
-    {
+    plain_vector U(p.mf_u().nb_dof());
+    if (!p.solve(U)) GMM_ASSERT1(false, "Solve has failed");
+    
+    { 
       getfem::mesh mcut;
       p.mls.global_cut_mesh(mcut);
       unsigned Q = p.mf_u().get_qdim();
@@ -797,13 +933,18 @@ int main(int argc, char *argv[]) {
       //	(getfem::fem_descriptor("FEM_PK_DISCONTINUOUS(2, 2, 0.0001)"));
       plain_vector V(mf.nb_dof());
 
+      /*for (unsigned i=0; i < p.mf_u().nb_dof(); ++i) {
+	cout << "dof " << i << ": " << p.mf_u().point_of_dof(i);
+      }
+      gmm::fill_random(U);*/
+
       getfem::interpolation(p.mf_u(), mf, U, V);
 
       getfem::stored_mesh_slice sl;
       getfem::mesh mcut_refined;
 
       unsigned NX = p.PARAM.int_value("NX"), nn;
-      if (NX < 6) nn = 24;
+      if (NX < 6) nn = 8;
       else if (NX < 12) nn = 8;
       else if (NX < 30) nn = 3;
       else nn = 1;
@@ -820,7 +961,7 @@ int main(int argc, char *argv[]) {
 	}
 
 	if (dmin < 1e-5)
-	  nrefine[cv] = nn*8;
+	  nrefine[cv] = nn*4;
 	else if (dmin < .1) 
 	  nrefine[cv] = nn*2;
 	else nrefine[cv] = nn;
@@ -882,8 +1023,8 @@ int main(int argc, char *argv[]) {
 			       p.PARAM.int_value("VTK_EXPORT")==1);
 
 	exp.exporting(mf_refined); 
-	exp.write_point_data(mf_refined_vm, DN, "error");
-	//exp.write_point_data(mf_refined_vm, VM, "von mises stress");
+	//exp.write_point_data(mf_refined_vm, DN, "error");
+	exp.write_point_data(mf_refined_vm, VM, "von mises stress");
 
 	exp.write_point_data(mf_refined, W, "elastostatic_displacement");
       
@@ -923,6 +1064,11 @@ int main(int argc, char *argv[]) {
 	   << getfem::asm_H1_dist(p.mim, p.mf_u(), U,
 				  p.exact_sol.mf, p.exact_sol.U) << "\n";
       
+       
+      cout << "L2 norm of the solution:"  << getfem::asm_L2_norm(p.mim,p.mf_u(),U)<<endl;
+      cout << "H1 norm of the solution:"  << getfem::asm_H1_norm(p.mim,p.mf_u(),U)<<endl;
+
+
       /* cout << "OLD ERROR L2:" 
 	 << getfem::asm_L2_norm(mim_refined,mf_refined,DIFF) 
 	 << " H1:" << getfem::asm_H1_dist(mim_refined,mf_refined,
@@ -935,7 +1081,7 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  DAL_STANDARD_CATCH_ERROR;
+  GMM_STANDARD_CATCH_ERROR;
 
   return 0; 
 }
