@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2002-2006 Yves Renard, Julien Pommier.
+// Copyright (C) 2002-2007 Yves Renard, Julien Pommier.
 //
 // This file is a part of GETFEM++
 //
@@ -33,15 +33,15 @@
    @see nonlinear_elastostatic.cc
 */
 
-#include <getfem_config.h>
-#include <getfem_assembling.h> /* import assembly methods (and norms comp.) */
-#include <getfem_export.h>   /* export functions (save solution in a file)  */
-#include <getfem_regular_meshes.h>
-#include <getfem_model_solvers.h>
-#include <gmm.h>
-#include <getfem_interpolation.h>
-#include <getfem_error_estimate.h>
-#include <getfem_import.h>
+#include "getfem/getfem_config.h"
+#include "getfem/getfem_assembling.h" /* import assembly methods (and norms comp.) */
+#include "getfem/getfem_export.h"   /* export functions (save solution in a file)  */
+#include "getfem/getfem_regular_meshes.h"
+#include "getfem/getfem_model_solvers.h"
+#include "gmm/gmm.h"
+#include "getfem/getfem_interpolation.h"
+#include "getfem/getfem_error_estimate.h"
+#include "getfem/getfem_import.h"
 
 /* some Getfem++ types that we will be using */
 using bgeot::base_small_vector; /* special class for small (dim<16) vectors */
@@ -211,7 +211,7 @@ struct elastostatic_problem {
   getfem::constraints_type dirichlet_version;
 
   std::string datafilename;
-  ftool::md_param PARAM;
+  bgeot::md_param PARAM;
 
   bool solve(plain_vector &U);
   void init(void);
@@ -247,8 +247,7 @@ void elastostatic_problem::init(void) {
   }
   getfem::import_mesh(filename.str(), mesh);
   
-  if (N != mesh.dim())
-    DAL_THROW(getfem::failure_error, "The mesh has not the right dimension");
+  GMM_ASSERT1(N == mesh.dim(), "The mesh has not the right dimension");
 
 #if GETFEM_PARA_LEVEL > 1
   cout<<"temps creation maillage "<< MPI_Wtime()-t_init<<endl;
@@ -304,11 +303,9 @@ void elastostatic_problem::init(void) {
      not used in the .param file */
   std::string data_fem_name = PARAM.string_value("DATA_FEM_TYPE");
   if (data_fem_name.size() == 0) {
-    if (!pf_u->is_lagrange()) {
-      DAL_THROW(dal::failure_error, "You are using a non-lagrange FEM. "
+    GMM_ASSERT1(pf_u->is_lagrange(), "You are using a non-lagrange FEM. "
 		<< "In that case you need to set "
 		<< "DATA_FEM_TYPE in the .param file");
-    }
     mf_rhs.set_finite_element(mesh.convex_index(), pf_u);
   } else {
     mf_rhs.set_finite_element(mesh.convex_index(), 
@@ -339,9 +336,9 @@ void elastostatic_problem::init(void) {
 
   cout<<"enumerate dof time "<< MPI_Wtime()-t_init<<endl;
 #else
-  double t_init = dal::uclock_sec();
+  double t_init = gmm::uclock_sec();
   mf_u.nb_dof(); mf_rhs.nb_dof(); mf_mult.nb_dof();
-  cout << "enumerate dof time " << dal::uclock_sec() - t_init << endl;
+  cout << "enumerate dof time " << gmm::uclock_sec() - t_init << endl;
 #endif
 }
 
@@ -451,7 +448,7 @@ bool elastostatic_problem::solve(plain_vector &U) {
       
       cout << "max = " << gmm::vect_norminf(ERR) << endl;
       // scalar_type threshold = gmm::vect_norminf(ERR) * 0.7;
-      scalar_type threshold = 0.01, min_ = 1e18;
+      scalar_type threshold = 0.0001, min_ = 1e18;
       cvref.clear();
       for (dal::bv_visitor i(mesh.convex_index()); !i.finished(); ++i) {
 	if (ERR[i] > threshold) cvref.add(i);
@@ -481,7 +478,7 @@ int main(int argc, char *argv[]) {
 
   GETFEM_MPI_INIT(argc, argv); // For parallelized version
 
-  DAL_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
+  GMM_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
   FE_ENABLE_EXCEPT;        // Enable floating point exception for Nan.
 
   try {
@@ -506,7 +503,7 @@ int main(int argc, char *argv[]) {
     t_ref=MPI_Wtime();
     cout<<"begining resol"<<endl;
 #endif
-    if (!p.solve(U)) DAL_THROW(dal::failure_error,"Solve has failed");
+    if (!p.solve(U)) GMM_ASSERT1(false, "Solve has failed");
 
 #if GETFEM_PARA_LEVEL > 1
     cout << "temps Resol "<< MPI_Wtime()-t_ref << endl;
@@ -532,10 +529,11 @@ int main(int argc, char *argv[]) {
     }
 
   }
-  DAL_STANDARD_CATCH_ERROR;
+  GMM_STANDARD_CATCH_ERROR;
 
   GETFEM_MPI_FINALIZE;
 
 
+
   return 0; 
 }
