@@ -1,7 +1,7 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Copyright (C) 2002-2006 Yves Renard, Julien Pommier.
+// Copyright (C) 2002-2007 Yves Renard, Julien Pommier.
 //
 // This file is a part of GETFEM++
 //
@@ -20,10 +20,10 @@
 //
 //========================================================================
 #include <numeric>
-#include <getfem_integration.h>
-#include <bgeot_comma_init.h>
-#include <getfem_mesh_fem.h>
-#include <getfem_mat_elem.h>
+#include "getfem/getfem_integration.h"
+#include "getfem/bgeot_comma_init.h"
+#include "getfem/getfem_mesh_fem.h"
+#include "getfem/getfem_mat_elem.h"
 #include <iomanip>
 #include <map>
 
@@ -50,12 +50,6 @@ void print_method(getfem::pintegration_method ppi) {
   cout << endl << endl;
 }
 
-class exception_cb : public dal::exception_callback  {
-  public:
-  virtual void callback(const std::string& msg)
-  { cerr << msg << endl; *(int *)(0) = 0; }
-};
-
 class matrix_collection {
 public:
   std::vector<base_vector> lst;
@@ -127,7 +121,8 @@ static void check_im_order(const std::string& s/*, size_type expected_pk=size_ty
       opt_long_scalar_type sum = 0, realsum = 1.;
       for (size_type i=0; i < ppi->approx_method()->nb_points_on_convex(); ++i) {
 	opt_long_scalar_type prod = ppi->approx_method()->coeff(i);
-	for (size_type d=0; d < dim; ++d) prod *= pow(ppi->approx_method()->point(i)[d], idx[d]);
+	for (size_type d=0; d < dim; ++d) 
+	  prod *= pow(opt_long_scalar_type(ppi->approx_method()->point(i)[d]), idx[d]);
 	sum += prod;
       }
       if (ppi->structure()->basic_structure() == bgeot::simplex_structure(dim)) {        
@@ -456,10 +451,10 @@ int main(/* int argc, char **argv */) {
     getfem::pintegration_method im_none = getfem::int_method_descriptor("IM_NONE()");
     try {
       cout << "nbpts=" << im_none->structure()->nb_points() << "\n";
-    } catch (dal::failure_error e) {
+    } catch (gmm::gmm_error e) {
       ok = 1;
     }
-    if (!ok) throw(dal::failure_error("IM_NONE failed..\n"));
+    if (!ok) throw(gmm::failure_error("IM_NONE failed..\n"));
     print_some_methods();
     check_methods();
     int failcnt = inspect_results();
@@ -467,7 +462,7 @@ int main(/* int argc, char **argv */) {
     //check_orders();
     if (failcnt) { cerr << "an error occured with " << failcnt << " integration methods\n"; return 1; }
   }
-  DAL_STANDARD_CATCH_ERROR;
+  GMM_STANDARD_CATCH_ERROR;
   
   return 0;
 
