@@ -1,8 +1,30 @@
-#include <getfem_assembling.h>
-#include <getfem_export.h>
-#include <getfem_regular_meshes.h>
-#include <getfem_mat_elem.h>
-#include <gmm.h>
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2007-2007 Yves Renard, Julien Pommier.
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
+#include "getfem/getfem_assembling.h"
+#include "getfem/getfem_assembling.h"
+#include "getfem/getfem_export.h"
+#include "getfem/getfem_regular_meshes.h"
+#include "getfem/getfem_mat_elem.h"
+#include "gmm/gmm.h"
 #ifdef GETFEM_HAVE_SYS_TIMES
 # include <sys/times.h>
 #endif
@@ -93,7 +115,7 @@ std::ostream& operator<<(std::ostream& o, const chrono& c) {
 }
 
 struct g_params {
-  ftool::md_param PARAM;
+  bgeot::md_param PARAM;
 
   size_type NX,Ndim;
   int mesh_type;
@@ -130,9 +152,10 @@ void g_params::init(int argc, char *argv[]) {
 
 namespace getfem {
   template<class VECT1, class VECT2>
-  void old_asm_Neumann_condition(VECT1 &B, const mesh_im &mim, const mesh_fem &mf,
-				 size_type boundary, const mesh_fem &mfdata, const VECT2 &F, dim_type N)
-  {
+  void old_asm_Neumann_condition(VECT1 &B, const mesh_im &mim,
+				 const mesh_fem &mf,
+				 size_type boundary, const mesh_fem &mfdata,
+				 const VECT2 &F, dim_type N) {
     size_type cv, nbd1, nbd2, f;
     dal::bit_vector nn = mf.convex_index(), nf;
     base_tensor t;
@@ -142,8 +165,8 @@ namespace getfem {
     pmat_elem_type pme; pmat_elem_computation pmec = 0;
 
     if (&(mf.linked_mesh()) != &(mfdata.linked_mesh()))
-      DAL_THROW(std::invalid_argument,
-		"This assembling procedure only works on a single mesh");
+      GMM_ASSERT1(false,
+		  "This assembling procedure only works on a single mesh");
   
     for (cv << nn; cv != ST_NIL; cv << nn) {
       nf = dal::bit_vector(mf.linked_mesh().region(boundary).faces_of_convex(cv));
@@ -171,7 +194,7 @@ namespace getfem {
 		  }
 		}
 	    }
-	  if (p != t.end()) DAL_THROW(dal::internal_error, "internal error"); 
+	  if (p != t.end()) GMM_ASSERT1(false, "internal error"); 
 	}
       }
     }
@@ -190,8 +213,8 @@ namespace getfem {
     pmat_elem_type pme; pmat_elem_computation pmec = 0;
 
     if (&(mf.linked_mesh()) != &(mfdata.linked_mesh()))
-      DAL_THROW(std::invalid_argument,
-		"This assembling procedure only works on a single mesh");
+      GMM_ASSERT1(false,
+		  "This assembling procedure only works on a single mesh");
 
     for (cv << nn; cv != ST_NIL; cv << nn)
       {
@@ -216,7 +239,7 @@ namespace getfem {
 		for (size_type k = 0; k < N; k++) B[dof1*N + k] += F[dof2*N+k]*(*p);
 	      }
 	  }
-	if (p != t.end()) DAL_THROW(dal::internal_error, "internal error"); 
+	if (p != t.end()) GMM_ASSERT1(false, "internal error"); 
       }
   }
 
@@ -235,8 +258,8 @@ namespace getfem {
     // M(0,0) = 1.0;  ??
 
     if (&(mf1.linked_mesh()) != &(mf2.linked_mesh()))
-      DAL_THROW(std::invalid_argument,
-		"This assembling procedure only works on a single mesh");
+      GMM_ASSERT1(false,
+		  "This assembling procedure only works on a single mesh");
 
     for (cv << nn; cv != ST_NIL; cv << nn)
       {
@@ -270,7 +293,7 @@ namespace getfem {
 		  M(dof1*N + k, dof2*N + k) += (*p);
 	      }
 	    }
-	    if (p != t.end()) DAL_THROW(dal::internal_error, "internal error"); 
+	    if (p != t.end()) GMM_ASSERT1(false, "internal error"); 
 	  }
 	}
       }
@@ -289,8 +312,8 @@ namespace getfem {
     // M(0,0) = 1.0;  ??
 
     if (&(mf1.linked_mesh()) != &(mf2.linked_mesh()))
-      DAL_THROW(std::invalid_argument,
-		"This assembling procedure only works on a single mesh");
+      GMM_ASSERT1(false,
+		  "This assembling procedure only works on a single mesh");
 
     for (cv << nn; cv != ST_NIL; cv << nn)
       {
@@ -321,7 +344,7 @@ namespace getfem {
 		  M(dof1*N + k, dof2*N + k) += (*p);
 	      }
 	  }
-	if (p != t.end()) DAL_THROW(dal::internal_error, "internal error"); 
+	if (p != t.end()) GMM_ASSERT1(false, "internal error"); 
       }
   }
 
@@ -340,8 +363,8 @@ namespace getfem {
     pmat_elem_type pme; pmat_elem_computation pmec = 0;
 
     if (&(mf_u.linked_mesh()) != &(mf_d.linked_mesh()))
-      DAL_THROW(std::invalid_argument,
-		"This assembling procedure only works on a single mesh");
+      GMM_ASSERT1(false,
+		  "This assembling procedure only works on a single mesh");
 
     for (cv << nn; cv != ST_NIL; cv << nn)
       {
@@ -409,7 +432,7 @@ namespace getfem {
 		    }
 		  }
 		}
-		if (p != t.end()) DAL_THROW(dal::internal_error, "internal error"); 
+		if (p != t.end()) GMM_ASSERT1(false, "internal error"); 
 	      }
 	  }
       }
@@ -432,8 +455,8 @@ namespace getfem {
     pmat_elem_type pme; pmat_elem_computation pmec = 0;
 
     if (&(mf.linked_mesh()) != &(mfdata.linked_mesh()))
-      DAL_THROW(std::invalid_argument,
-		"This assembling procedure only works on a single mesh");
+      GMM_ASSERT1(false,
+		  "This assembling procedure only works on a single mesh");
   
     for (cv << nn; cv != ST_NIL; cv << nn)
       {
@@ -491,7 +514,7 @@ namespace getfem {
 		      }
 		}
 	  }
-	if (p != t.end()) DAL_THROW(dal::internal_error, "internal error"); 
+	if (p != t.end()) GMM_ASSERT1(false, "internal error"); 
       }
   }
 
@@ -518,9 +541,9 @@ namespace getfem {
 
     if (&(mf_u.linked_mesh()) != &(mf_p.linked_mesh())
 	|| &(mf_u.linked_mesh()) != &(mf_d.linked_mesh()))
-      DAL_THROW(std::invalid_argument,
-		"This assembling procedure only works on a single mesh");
-
+      GMM_ASSERT1(false,
+		  "This assembling procedure only works on a single mesh");
+    
     /* loop over all convexes */
     for (cv << nn; cv != ST_NIL; cv << nn) {
 
@@ -569,7 +592,7 @@ namespace getfem {
 	  } 
 	}
       }
-      if (p != t.end()) DAL_THROW(dal::internal_error, "internal error"); 
+      if (p != t.end()) GMM_ASSERT1(false, "internal error"); 
     }
   }
 
@@ -586,8 +609,8 @@ namespace getfem {
     pmat_elem_computation pmec = 0;
 
     if (&(mf.linked_mesh()) != &(mfdata.linked_mesh()))
-      DAL_THROW(std::invalid_argument,
-		"This assembling procedure only works on a single mesh");
+      GMM_ASSERT1(false,
+		  "This assembling procedure only works on a single mesh");
 
     for (cv << nn; cv != ST_NIL; cv << nn)
       {
@@ -622,7 +645,7 @@ namespace getfem {
 	    }
 	  }
 	}
-	if (p != t.end()) DAL_THROW(dal::internal_error, "internal error"); 
+	if (p != t.end()) GMM_ASSERT1(false, "internal error"); 
       }
   }
 
@@ -731,10 +754,10 @@ namespace getfem {
                                                          const VECT &LAMBDA,const VECT &MU) {
     MAT &RM = const_cast<MAT &>(RM_);
     if (mfdata.get_qdim() != 1)
-      DAL_THROW(invalid_argument, "invalid data mesh fem (Qdim=1 required)");
+      GMM_ASSERT1(false, "invalid data mesh fem (Qdim=1 required)");
     
     if (mf.get_qdim() != mf.linked_mesh().dim())
-      DAL_THROW(std::logic_error, "wrong qdim for the mesh_fem");
+      GMM_ASSERT1(false, "wrong qdim for the mesh_fem");
     generic_assembly assem("lambda=data$1(#2); mu=data$2(#2);"
 			   "M(#1,#1)+= sym(comp(vGrad(#1)(:,i,j).vGrad(#1)(:,i,j).Base(#2)(k).mu(k)) +"
 			   "               comp(vGrad(#1)(:,j,i).vGrad(#1)(:,i,j).Base(#2)(k).mu(k)) +"
@@ -807,7 +830,7 @@ void gen_mesh(getfem::mesh& mesh) {
 		     (mesh, param.Ndim, org, vtab.begin(), ref.begin()); 
     cerr << mesh.convex_index().card() << " " << param.Ndim << "D prisms generated\n";
     break;
-  default : DAL_THROW(dal::internal_error, "Unknown type of mesh");
+  default : GMM_ASSERT1(false, "Unknown type of mesh");
   }
 
   assert(param.NX>2);
@@ -1116,7 +1139,7 @@ void tensor_ref_check3(unsigned n1, unsigned n2, unsigned n3, unsigned n4, unsig
     for (unsigned i=0; i < n1; ++i) {
       if (gmm::abs(trABCD.base()[i+j*n1] - ABCD(i,j)) > 1e-10) {
 	cerr << "FAILED : " << i << ", " << j << ", " << trABCD.base()[i+j*n1] << "!=" << ABCD(i,j) << "\n";
-	DAL_INTERNAL_ERROR("");
+	GMM_ASSERT1(false, "");
       }
     }
 }
@@ -1556,7 +1579,16 @@ void test_matrixfem(const getfem::mesh_im &mim) {
 }
 
 
-void inline_red_test(const getfem::mesh_im &mim, getfem::mesh_fem &mf1, getfem::mesh_fem &mf2) {
+
+
+
+void inline_red_test(const getfem::mesh_im &mim, getfem::mesh_fem &mf1_, getfem::mesh_fem &mf2) {
+  std::stringstream ss; mf1_.write_to_file(ss);
+  getfem::mesh_fem mf1(mf1_.linked_mesh()); mf1.read_from_file(ss);
+
+  if (mf1_.convex_index().is_in(2))
+    mf1.set_finite_element(2, getfem::fem_descriptor("FEM_PK(2,1)"));
+
   std::vector<scalar_type> U(mf1.nb_dof()); gmm::fill_random(U);
   std::vector<scalar_type> V(mf2.nb_dof()); gmm::fill_random(V);
   getfem::generic_assembly assem;    
@@ -1572,47 +1604,143 @@ void inline_red_test(const getfem::mesh_im &mim, getfem::mesh_fem &mf1, getfem::
   assem.push_vec(v1);
   assem.assembly();
   
-  cout << "OLD SCHOOL\n";
-  getfem::generic_assembly assem2;
-  assem2.set("u=data(#1);v=data$2(#2);"
-	     "V()+=u(i).v(j).comp(Grad(#1).Grad(#2))(i,d,j,d);"); //print comp(Grad(#1).Grad(#2))(:,d,:,d)");
+  {
+    cout << "OLD SCHOOL\n";
+    getfem::generic_assembly assem2;
+    assem2.set("u=data(#1);v=data$2(#2);"
+	       "V()+=u(i).v(j).comp(Grad(#1).Grad(#2))(i,d,j,d);"); //print comp(Grad(#1).Grad(#2))(:,d,:,d)");
+    assem2.push_mi(mim);
+    assem2.push_mf(mf1);
+    assem2.push_mf(mf2);
+    assem2.push_data(U);
+    assem2.push_data(V);
+    std::vector<scalar_type> v2(1);
+    assem2.push_vec(v2);
+    assem2.assembly();
+
+
+
+    cout << "v1 = " << v1 << ", v2 = " << v2 << endl;
+    assert(gmm::abs(v1[0]-v2[0]) < 1e-14);
+  }
+  {
+    cout << "INLINE RED2\n";
+    getfem::generic_assembly assem3;
+    mf2.set_qdim(1);
+    mf1.set_qdim(1);
+    U.resize(mf1.nb_dof());
+    V.resize(mf2.nb_dof());
+    assem3.set("u=data(#1);v=data$2(#2);"
+	       //"V()+=comp(vGrad(#1)(i,j,k).u(i));");
+	       "V()+=comp(vGrad(#1)(i,k,:).vGrad(#2)(j,k,:).v(j).u(i))(d,d);");
+    /*
+      assem3.set("u=data(#1);"
+      "t=comp(vGrad(#1)(k,:,:).vGrad(#1)(l,:,:).vGrad(#2).u(k).u(l));"
+      "V()+=t(i,j,k,l,m,n,o);");
+    */
+    assem3.push_mi(mim);
+    assem3.push_mf(mf1);
+    assem3.push_mf(mf2);
+    assem3.push_data(U);
+    assem3.push_data(V);
+    std::vector<scalar_type> v3(1);
+    assem3.push_vec(v3);
+    assem3.assembly();
+    mf1.set_qdim(1);
+    mf2.set_qdim(1);
+    
+    
+    cout << "v1 = " << v1 << ", v3 = " << v3 << endl;
+
+    mf1.write_to_file(std::cerr);
+    
+    assert(gmm::abs(v1[0]-v3[0]) < 1e-14);
+  }
+}
+
+template<typename VECT1> class shape_der_nonlinear_term 
+  : public getfem::nonlinear_elem_term {
+  
+  const getfem::mesh_fem &mf;
+  const VECT1 &U;
+  size_type N;
+  base_vector coeff;
+  base_matrix gradU, E, Sigma;
+  bgeot::multi_index sizes_;
+  scalar_type lambda, mu;
+  
+public:
+  shape_der_nonlinear_term(const getfem::mesh_fem &mf_, const VECT1 &U_,
+			  scalar_type lambda_, scalar_type mu_) 
+    : mf(mf_), U(U_),
+      N(mf_.get_qdim()),
+      gradU(N, N), E(N, N), Sigma(N,N), sizes_(N,N),
+      lambda(lambda_), mu(mu_) { }
+  
+  const bgeot::multi_index &sizes() const { return sizes_; }
+  
+  virtual void compute(getfem::fem_interpolation_context& ,
+		       bgeot::base_tensor &t) {
+    assert(t.size() == N*N);
+    /*size_type cv = ctx.convex_num();
+    coeff.resize(mf.nb_dof_of_element(cv));
+    gmm::copy(gmm::sub_vector(U, gmm::sub_index(mf.ind_dof_of_element(cv))),
+	      coeff);
+    ctx.pf()->interpolation_grad(ctx, coeff, gradU, mf.get_qdim());
+    gmm::copy(gradU, E); gmm::add(gmm::transposed(gradU), E);
+    gmm::scale(E, 0.5);
+    gmm::copy(gmm::identity_matrix(), Sigma);
+    gmm::scale(Sigma, lambda * gmm::mat_trace(E));
+    gmm::add(gmm::scaled(E, 2.*mu), Sigma);
+    
+    for (size_type i = 0; i < N; ++i) 
+      for (size_type j = 0; j < N; ++j) {
+	t(i,j) = 0.0;
+	for (size_type k = 0; k < N; ++k)
+	  t(i,j) += Sigma(i,k) * gradU(k,j); 
+      }
+    */
+    for (size_type i = 0; i < N; ++i) 
+      for (size_type j = 0; j < N; ++j)
+	t(i,j) = 0.0;
+    
+  }
+};
+
+void testbug() {
+  std::vector<size_type> nsubdiv(3); nsubdiv[0] = nsubdiv[1] = nsubdiv[2] = 3;
+  getfem::mesh m; 
+  getfem::regular_unit_mesh(m, nsubdiv, bgeot::simplex_geotrans(3,1));
+  getfem::mesh_fem mf1(m,3), mf2(m,3);
+  mf1.set_classical_finite_element(m.convex_index(), 1);
+  mf2.set_classical_finite_element(m.convex_index(), 1);
+  /*mf1.set_finite_element(12, getfem::fem_descriptor("FEM_PK(3,2)"));
+    mf2.set_finite_element(15, getfem::fem_descriptor("FEM_PK(3,2)"));*/
+  getfem::mesh_im mim(m); mim.set_integration_method(m.convex_index(), 5);
+  std::vector<scalar_type> U(mf1.nb_dof()), SD(mf2.nb_dof());
+  gmm::fill_random(U);
+
+  shape_der_nonlinear_term<std::vector<scalar_type> > nl(mf1, U, 0, 1);
+  
+  // trigers a real bug: printing an empty (because of the "vectorization" of vBase)
+  // subtensor will crash
+  getfem::generic_assembly assem2
+    ("t=comp(vBase(#1).vBase(#2));"
+     "print(t(:,:,2,3)); ");
+
+  
+
+  //"V(#2) += t(:,:,2,3)"); //0.5*t(i,i,:,j,j) - t(i,j,:,j,i);");
   assem2.push_mi(mim);
   assem2.push_mf(mf1);
+  assem2.push_nonlinear_term(&nl);
   assem2.push_mf(mf2);
-  assem2.push_data(U);
-  assem2.push_data(V);
-  std::vector<scalar_type> v2(1);
-  assem2.push_vec(v2);
+  assem2.push_vec(SD);
+  
+  double t0 = gmm::uclock_sec();
   assem2.assembly();
-
-
-
-  cout << "v1 = " << v1 << ", v2 = " << v2 << endl;
-  assert(gmm::abs(v1[0]-v2[0]) < 1e-14);
-
-  cout << "INLINE RED2\n";
-  getfem::generic_assembly assem3;
-  mf2.set_qdim(1);
-  mf1.set_qdim(1);
-  U.resize(mf1.nb_dof());
-  V.resize(mf2.nb_dof());
-  assem3.set("u=data(#1);v=data$2(#2);"
-	     //"V()+=comp(vGrad(#1)(i,j,k).u(i));");
-	     "V()+=comp(vGrad(#1)(i,k,d).vGrad(#2)(j,k,d).v(j).u(i));");
-  assem3.push_mi(mim);
-  assem3.push_mf(mf1);
-  assem3.push_mf(mf2);
-  assem3.push_data(U);
-  assem3.push_data(V);
-  std::vector<scalar_type> v3(1);
-  assem3.push_vec(v3);
-  assem3.assembly();
-  mf1.set_qdim(1);
-  mf2.set_qdim(1);
-
-
-  cout << "v1 = " << v1 << ", v3 = " << v3 << endl;
-  assert(gmm::abs(v1[0]-v3[0]) < 1e-14);
+  cerr << " done : " << gmm::uclock_sec() - t0 << "\n"; exit(1);
+  exit(1);
 }
 
 void test_gradgt(const getfem::mesh_im &mim, const getfem::mesh_fem &mf1) {
@@ -1644,7 +1772,7 @@ void test_gradgt(const getfem::mesh_im &mim, const getfem::mesh_fem &mf1) {
 
 int main(int argc, char *argv[]) {
 
-  DAL_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
+  GMM_SET_EXCEPTION_DEBUG; // Exceptions make a memory fault, to debug.
   FE_ENABLE_EXCEPT;        // Enable floating point exception for Nan.
 
   try {
@@ -1652,6 +1780,7 @@ int main(int argc, char *argv[]) {
     // getfem::pfem pf = getfem::fem_descriptor("FEM_PK_PRISM_HIERARCHICAL(3,3)");
     
 
+    //testbug();
     do_general_check();
 
 #ifdef ASSEMBLY_CHECK
@@ -1663,6 +1792,7 @@ int main(int argc, char *argv[]) {
       tests[param.do_what]=true;
     }
 
+
    cerr << "\n\n-----------------------------SIMPLE MESH TESTS---------------------\n\n";
    {
      getfem::mesh m(2);
@@ -1743,7 +1873,7 @@ int main(int argc, char *argv[]) {
    }
 #endif /* ASSEMBLY_CHECK */
   }
-  DAL_STANDARD_CATCH_ERROR;
+  GMM_STANDARD_CATCH_ERROR;
   cout << "failures: " << fail_cnt << endl;
   return fail_cnt; 
 }
